@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: codecs.h,v $
+ * Revision 1.66  2002/12/16 09:11:15  robertj
+ * Added new video bit rate control, thanks Walter H. Whitlock
+ *
  * Revision 1.65  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -885,26 +888,57 @@ class H323VideoCodec : public H323Codec
      */ 
     int GetHeight() const { return frameHeight; }
 
-    
     /**Quality of the transmitted video. 1 is good, 31 is poor.
      */
-    virtual void SetTxQualityLevel(int qlevel) {qualityLevel = qlevel; }
+    virtual void SetTxQualityLevel(int qlevel) {videoQuality = qlevel; }
+
+    /**Minimum quality limit for the transmitted video.
+     * Default is 1.  Encode quality will not be set below this value.
+     */
+    virtual void SetTxMinQuality(int qlevel) {videoQMin = qlevel; }
+
+    /**Maximum quality limit for the transmitted video.
+     * Default is 24.  Encode quality will not be set above this value.
+     */
+    virtual void SetTxMaxQuality(int qlevel) {videoQMax = qlevel; }
 
     /**number of blocks (that haven't changed) transmitted with each 
        frame. These blocks fill in the background */
     virtual void SetBackgroundFill(int idle) {fillLevel= idle; }
  
     enum BitRateModeBits {
+      None                = 0x00,
       DynamicVideoQuality = 0x01,
       AdaptivePacketDelay = 0x02
     };
 
-    /**Set average bitrate when transmitting video.
-       A value of 0 disables bit rate control.
+    /**Get the current value for video control mode
      */
-    virtual BOOL SetAverageBitRate(
-      unsigned bitRate,   // New bit rate
-      unsigned modes = DynamicVideoQuality
+    unsigned GetVideoMode(void) {return videoBitRateControlModes;}
+
+    /**Set the current value for video control mode
+     * return the resulting value video control mode
+     */
+    unsigned SetVideoMode(unsigned mode) {return videoBitRateControlModes = mode;}
+    
+    /**Set maximum bitrate when transmitting video.  A value of 0 disables bit rate
+       control.  The average bitrate will be less depending on channel dead time,
+       i.e. time that the channel could be transmitting bits but is not.
+     */
+    virtual BOOL SetMaxBitRate(
+      unsigned bitRate // New bit rate
+    );
+
+    /**Set target time in milliseconds between video frames going through
+       the channel.  This sets the video frame rate through the channel,
+       which is <= grabber frame rate.  Encoder quality will be adjusted
+       dynamically by the codec to find a frame size that allows sending
+       at this rate.  Default = 167 ms = 6 frames per second.  A value of 0
+       means the channel will attempt to run at the video grabber frame rate
+       Sometimes the channel cannot transmit as fast as the video grabber.
+    */
+    virtual BOOL SetTargetFrameTimeMs(
+      unsigned ms // new time between frames
     );
 
     /**
@@ -915,28 +949,28 @@ class H323VideoCodec : public H323Codec
  
   protected:
 
-    int     frameWidth;
-    int     frameHeight;
+    int frameWidth;
+    int frameHeight;
+    int fillLevel;
 
-    int     qualityLevel;
-    int     fillLevel;
-
-    float estFps;                           
-    int frameRateDivider;                   
-    int defBitsPerFrame;                    
-    int reqBitsPerFrame;                    
-    int frameBits;                          
-    int numOf31, numOfNot31;                
-    int highLimit, lowLimit, actualQuality; 
-    int frameNum;    
-    int videoBitRate;
+    // used in h261codec.cxx
     unsigned videoBitRateControlModes;
-    PTimeInterval timeFor5Frames;           
-
-    int bytesSent;
-    PTimeInterval startTime;
-    PTimeInterval lastReadReturn;
-
+    // variables used for video bit rate control
+    int bitRateHighLimit; // maximum instantaneous bit rate allowed
+    unsigned oldLength;
+    PTimeInterval oldTime;
+    PTimeInterval newTime;
+    // variables used for dynamic video quality control
+    int targetFrameTimeMs; //targetFrameTimeMs = 1000 / videoSendFPS
+    int frameBytes; // accumulate count of bytes per frame
+    int sumFrameTimeMs, sumAdjFrameTimeMs, sumFrameBytes; // accumulate running average
+    int videoQMax, videoQMin; // dynamic video quality min/max limits
+    int videoQuality; // current video encode quality setting, 1..31
+    PTimeInterval frameStartTime;
+    PTimeInterval grabInterval;
+    
+    int frameNum, packetNum, oldPacketNum;
+    
     PMutex  videoHandlerActive;    
 };
 
