@@ -27,6 +27,54 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: gkserver.h,v $
+ * Revision 1.52  2002/12/17 01:25:23  robertj
+ * Added call backs on gk server when ep indicates alerting or connect.
+ *
+ * Revision 1.51  2002/11/28 05:45:46  robertj
+ * Fixed bug so can set total bandwidth while calls are in progress.
+ *
+ * Revision 1.50  2002/11/28 04:41:44  robertj
+ * Added support for RAS ServiceControlIndication command.
+ *
+ * Revision 1.49  2002/11/27 06:54:52  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.48  2002/11/22 00:11:29  robertj
+ * Added call start time.
+ *
+ * Revision 1.47  2002/11/18 23:40:45  robertj
+ * Changed to use the H323EndPoint::CreateAuthenticators() function, we
+ *   should use only one.
+ *
+ * Revision 1.46  2002/11/12 11:36:03  robertj
+ * Added function to find endpoint by partial alias.
+ * Optimised endpoint "indexes" to use sorted lists instead of dictionaries.
+ *
+ * Revision 1.45  2002/11/06 23:23:48  robertj
+ * Fixed minor error in parameter, should be reference not value
+ *
+ * Revision 1.44  2002/10/29 00:12:02  robertj
+ * Changed template classes so things like PSafeList actually creates the
+ *   base collection class as well.
+ * Improved setting of usage info, included getting it from BRQ etc.
+ *
+ * Revision 1.43  2002/10/21 00:54:47  robertj
+ * Added function to unregister an endpoint via user interface.
+ * Added reason code to call disengage.
+ * Added ability to find call by a description string.
+ *
+ * Revision 1.42  2002/10/17 05:26:33  robertj
+ * Added function to get at registered endpoints protocol version.
+ *
+ * Revision 1.41  2002/10/16 07:22:50  robertj
+ * Added support for endpoints that do not support RRQ timeToLive parameter,
+ *   will actively go see if ep is there using IRQ before expiring the ep.
+ *
  * Revision 1.40  2002/10/01 08:17:34  robertj
  * Fixed (benign) race condition between client and server both wanting to do
  *   a DRQ of a call at the same time. Caused an Assert, but no crash.
@@ -195,6 +243,7 @@ class H225_ArrayOf_TransportAddress;
 class H225_GatekeeperIdentifier;
 class H225_EndpointIdentifier;
 class H225_InfoRequestResponse_perCallInfo_subtype;
+class H225_RasUsageInformation;
 
 class H323RegisteredEndPoint;
 class H323GatekeeperListener;
@@ -598,7 +647,9 @@ class H323GatekeeperCall : public PSafeObject
     /**Shut down a call.
        This sendsa DRQ to the endpoint(s) to close the call down.
       */
-    virtual BOOL Disengage();
+    virtual BOOL Disengage(
+      int reason = -1   // Reason for disengage
+    );
 
     /**Handle a disengage DRQ PDU.
        The default behaviour simply returns TRUE.
@@ -623,6 +674,14 @@ class H323GatekeeperCall : public PSafeObject
       H225_InfoRequestResponse_perCallInfo_subtype & call
     );
 
+    /**Call back when an info response detects an Alerting.
+      */
+    virtual void OnAlerting();
+
+    /**Call back when an info response detects an Connected.
+      */
+    virtual void OnConnected();
+
     /**Function called to do heartbeat check of the call.
        Monitor the state of the call and make sure everything is OK.
 
@@ -634,6 +693,51 @@ class H323GatekeeperCall : public PSafeObject
        still there and running. If the IRQ fails, FALSE is returned.
       */
     virtual BOOL OnHeartbeat();
+
+    /**Get the current credit for this call.
+       This function is only called if the client indicates that it can use
+       the information provided.
+
+       The default behaviour calls the same function on the endpoint.
+      */
+    virtual PString GetCallCreditAmount() const;
+
+    /**Get the call credit billing mode for this endpoint.
+       This function is only called if the client indicates that it can use
+       the information provided.
+
+       The default behaviour calls the same function on the endpoint.
+      */
+    virtual BOOL GetCallCreditMode() const;
+
+    /**Get the duration limit for this call.
+       This function is only called if the client indicates that it can use
+       the information provided.
+
+       The default behaviour returns zero which indicates there is no
+       duration limit applicable.
+      */
+    virtual unsigned GetDurationLimit() const;
+
+    /**Send the call credit service control PDU.
+       This will send an SCI pdu to the endpoint with the control service
+       session information for the current call credit, if enabled.
+      */
+    virtual BOOL SendCallCreditServiceControl();
+
+    /**Add call credit and duration information to PDU.
+      */
+    BOOL AddCallCreditServiceControl(
+      H225_ArrayOf_ServiceControlSession & serviceControl
+    ) const;
+
+    /**Send the service control session for the PDU.
+       This will send an SCI pdu to the endpoint with the control service
+       session information provided.
+      */
+    virtual BOOL SendServiceControlSession(
+      const H323ServiceControlSession & session
+    );
   //@}
 
   /**@name Access functions */
@@ -655,6 +759,7 @@ class H323GatekeeperCall : public PSafeObject
     unsigned GetBandwidthUsed() const { return bandwidthUsed; }
     void SetBandwidthUsed(unsigned bandwidth) { bandwidthUsed = bandwidth; }
     const PTime & GetLastInfoResponseTime() const { return lastInfoResponse; }
+    const PTime & GetCallStartTime() const { return callStartTime; }
     const PTime & GetAlertingTime() const { return alertingTime; }
     const PTime & GetConnectedTime() const { return connectedTime; }
     const PTime & GetCallEndTime() const { return callEndTime; }
@@ -662,6 +767,8 @@ class H323GatekeeperCall : public PSafeObject
   //@}
 
   protected:
+    void SetUsageInfo(const H225_RasUsageInformation & usage);
+
     H323GatekeeperServer   & gatekeeper;
     H323RegisteredEndPoint * endpoint;
     H323GatekeeperListener * rasChannel;
@@ -681,6 +788,7 @@ class H323GatekeeperCall : public PSafeObject
     PTime                lastInfoResponse;
 
     BOOL                          drqReceived;
+    PTime                         callStartTime;
     PTime                         alertingTime;
     PTime                         connectedTime;
     PTime                         callEndTime;
@@ -688,9 +796,6 @@ class H323GatekeeperCall : public PSafeObject
 };
 
 
-PSORTED_LIST(H323GatekeeperCallList, H323GatekeeperCall);
-
-
 /**This class describes endpoints that are registered with a gatekeeper server.
    Note that a registered endpoint has no realationship in this software to a
    H323EndPoint class. This is purely a description of endpoints that are
@@ -798,6 +903,13 @@ class H323RegisteredEndPoint : public PSafeObject
       H323GatekeeperURQ & request
     );
 
+    /**Force unregistration of the endpoint.
+       This sendsa URQ to the endpoint(s) to close the call down.
+      */
+    virtual BOOL Unregister(
+      int reason = -1   // Reason for unregistration
+    );
+
     /**Handle an info request response IRR PDU.
        The default behaviour finds each call current for endpoint and calls 
        the function of the same name in the H323GatekeeperCall instance.
@@ -806,19 +918,65 @@ class H323RegisteredEndPoint : public PSafeObject
       H323GatekeeperIRR & request
     );
 
+    /**Function called to do time to live check of the call.
+       Monitor the state of the endpoint and make sure everything is OK.
+
+       A return value of FALSE indicates the endpoint has expired and is to be
+       unregistered and removed.
+
+       Default behaviour checks the time since the last received RRQ and if
+       it has been too long does an IRQ to see if the endpoint is
+       still there and running. If the IRQ fails, FALSE is returned.
+      */
+    virtual BOOL OnTimeToLive();
+
+    /**Get the current call credit for this endpoint.
+       This function is only called if the client indicates that it can use
+       the information provided. If a server wishes to enable this feature by
+       returning a non-empty string, it must be consistent in that usage. That
+       is the H323GatekeeperCall::GetCallCreditAmount() for this endpoint
+       must also return non-empty value.
+
+       The return value is a UTF-8 string for amount, including currency.
+
+       The default behaviour returns an empty string disabling the function.
+      */
+    virtual PString GetCallCreditAmount() const;
+
+    /**Get the call credit billing mode for this endpoint.
+       This function is only called if the client indicates that it can use
+       the information provided.
+
+       The default behaviour return TRUE indicating that calls will debit the
+       account.
+      */
+    virtual BOOL GetCallCreditMode() const;
+
+    /**Send the service control session for the PDU.
+       This will send an SCI pdu to the endpoint with the control service
+       session information provided.
+      */
+    virtual BOOL SendServiceControlSession(
+      const H323ServiceControlSession & session
+    );
+
+    /**Set the service control session for the PDU.
+       This is an internal function.
+      */
+    virtual BOOL AddServiceControlSession(
+      const H323ServiceControlSession & session,
+      H225_ArrayOf_ServiceControlSession & serviceControl
+    );
+  //@}
+
+  /**@name Access functions */
+  //@{
     /**Set password for user activating H.235 security.
       */
     virtual BOOL SetPassword(
       const PString & password
     );
 
-    /**Determine of this endpoint has exceeded its time to live.
-      */
-    BOOL HasExceededTimeToLive() const;
-  //@}
-
-  /**@name Access functions */
-  //@{
     /**Get the endpoint identifier assigned to the endpoint.
       */
     const PString & GetIdentifier() const { return identifier; }
@@ -888,12 +1046,24 @@ class H323RegisteredEndPoint : public PSafeObject
 
     /**Get application info (name/version etc) for endpoint.
       */
-    const PCaselessString GetApplicationInfo() const { return applicationInfo; }
+    const PCaselessString & GetApplicationInfo() const { return applicationInfo; }
 
+    /**Get the protocol version the endpoint registered with.
+      */
+    unsigned GetProtocolVersion() const { return protocolVersion; }
+
+    /**Get the flag indicating the endpoint can display credit amounts.
+      */
+    BOOL CanDisplayAmountString() const { return canDisplayAmountString; }
+
+    /**Get the flag indicating the endpoint can enforce a duration limit.
+      */
+    BOOL CanEnforceDurationLimit() const { return canEnforceDurationLimit; }
   //@}
 
   protected:
     H323GatekeeperServer    & gatekeeper;
+    H323GatekeeperListener  * rasChannel;
 
     PString                   identifier;
     H323TransportAddressArray rasAddresses;
@@ -902,11 +1072,16 @@ class H323RegisteredEndPoint : public PSafeObject
     PStringArray              voicePrefixes;
     PCaselessString           applicationInfo;
     unsigned                  protocolVersion;
+    BOOL                      canDisplayAmountString;
+    BOOL                      canEnforceDurationLimit;
     unsigned                  timeToLive;
     H235Authenticators        authenticators;
 
-    PTime                  lastRegistration;
-    H323GatekeeperCallList activeCalls;
+    PTime lastRegistration;
+    PTime lastInfoResponse;
+
+    PSortedList<H323GatekeeperCall> activeCalls;
+    POrdinalDictionary<PString>     serviceControlSessions;
 };
 
 
@@ -937,6 +1112,13 @@ class H323GatekeeperListener : public H225_RAS
 
   /**@name Operations */
   //@{
+    /**Send a UnregistrationRequest (URQ) to endpoint.
+      */
+    BOOL UnregistrationRequest(
+      const H323RegisteredEndPoint & ep,
+      unsigned reason
+    );
+
     /**Send a DisengageRequest (DRQ) to endpoint.
       */
     BOOL DisengageRequest(
@@ -950,6 +1132,14 @@ class H323GatekeeperListener : public H225_RAS
       H323RegisteredEndPoint & ep,
       H323GatekeeperCall * call = NULL
     );
+
+    /**Send an ServiceControlIndication (SCI) to endpoint.
+      */
+    virtual BOOL ServiceControlIndication(
+      H323RegisteredEndPoint & ep,
+      const H323ServiceControlSession & session,
+      H323GatekeeperCall * call = NULL
+    );
   //@}
 
   /**@name Operation callbacks */
@@ -1230,41 +1420,48 @@ class H323GatekeeperServer : public PObject
       */
     virtual PString CreateEndPointIdentifier();
 
-    /**Find a registered alias given its endpoint identifier.
+    /**Find a registered endpoint given its endpoint identifier.
       */
     virtual PSafePtr<H323RegisteredEndPoint> FindEndPointByIdentifier(
       const PString & identifier,
       PSafetyMode mode = PSafeReference
     );
 
-    /**Find a registered alias given a list of signal addresses.
+    /**Find a registered endpoint given a list of signal addresses.
       */
     virtual PSafePtr<H323RegisteredEndPoint> FindEndPointBySignalAddresses(
       const H225_ArrayOf_TransportAddress & addresses,
       PSafetyMode mode = PSafeReference
     );
 
-    /**Find a registered alias given its signal address.
+    /**Find a registered endpoint given its signal address.
       */
     virtual PSafePtr<H323RegisteredEndPoint> FindEndPointBySignalAddress(
       const H323TransportAddress & address,
       PSafetyMode mode = PSafeReference
     );
 
-    /**Find a registered alias given its raw alias address.
+    /**Find a registered endpoint given its raw alias address.
       */
     virtual PSafePtr<H323RegisteredEndPoint> FindEndPointByAliasAddress(
       const H225_AliasAddress & alias,
       PSafetyMode mode = PSafeReadWrite
     );
 
-    /**Find a registered alias given its simple alias string.
+    /**Find a registered endpoint given its simple alias string.
       */
     virtual PSafePtr<H323RegisteredEndPoint> FindEndPointByAliasString(
       const PString & alias,
       PSafetyMode mode = PSafeReference
     );
 
+    /**Find the first registered endpoint given a partial alias string.
+      */
+    virtual PSafePtr<H323RegisteredEndPoint> FindEndPointByPartialAlias(
+      const PString & alias,
+      PSafetyMode mode = PSafeReference
+    );
+
     /**Get first endpoint for enumeration.
       */
     PSafePtr<H323RegisteredEndPoint> GetFirstEndPoint(
@@ -1318,6 +1515,13 @@ class H323GatekeeperServer : public PObject
       H323GatekeeperCall * call
     );
 
+    /**Find the call given the identifier.
+      */
+    virtual PSafePtr<H323GatekeeperCall> FindCall(
+      const PString & description,
+      PSafetyMode mode = PSafeReference
+    );
+
     /**Find the call given the identifier.
       */
     virtual PSafePtr<H323GatekeeperCall> FindCall(
@@ -1330,7 +1534,7 @@ class H323GatekeeperServer : public PObject
       */
     virtual PSafePtr<H323GatekeeperCall> FindCall(
       const OpalGloballyUniqueID & callIdentifier,
-      H323GatekeeperCall::Direction direction = H323GatekeeperCall::UnknownDirection,
+      H323GatekeeperCall::Direction direction,
       PSafetyMode mode = PSafeReference
     );
 
@@ -1444,10 +1648,6 @@ class H323GatekeeperServer : public PObject
 
   /**@name Security and authentication functions */
   //@{
-    /**Get the default list of authenticators for endpoint.
-      */
-    virtual H235Authenticators CreateAuthenticators() const;
-
     /**Get separate H.235 authentication for the connection.
        This allows an individual ARQ to override the authentical credentials
        used in H.235 based RAS for this particular connection.
@@ -1477,6 +1677,10 @@ class H323GatekeeperServer : public PObject
 
   /**@name Access functions */
   //@{
+    /**Get the owner endpoint.
+     */
+    H323EndPoint & GetOwnerEndPoint() const { return ownerEndPoint; }
+
     /**Get the identifier name for this gatekeeper.
       */
     const PString & GetGatekeeperIdentifier() const { return gatekeeperIdentifier; }
@@ -1493,11 +1697,15 @@ class H323GatekeeperServer : public PObject
 
     /**Get the total bandwidth available in 100's of bits per second.
       */
-    unsigned GetAvailableBandwidth() const { return availableBandwidth; }
+    unsigned GetAvailableBandwidth() const { return totalBandwidth; }
 
     /**Set the total bandwidth available in 100's of bits per second.
       */
-    void SetAvailableBandwidth(unsigned bps100) { availableBandwidth = bps100; }
+    void SetAvailableBandwidth(unsigned bps100) { totalBandwidth = bps100; }
+
+    /**Get the total bandwidth used in 100's of bits per second.
+      */
+    unsigned GetUsedBandwidth() const { return usedBandwidth; }
 
     /**Get the default bandwidth for calls.
       */
@@ -1557,7 +1765,8 @@ class H323GatekeeperServer : public PObject
 
     // Configuration & policy variables
     PString  gatekeeperIdentifier;
-    unsigned availableBandwidth;
+    unsigned totalBandwidth;
+    unsigned usedBandwidth;
     unsigned defaultBandwidth;
     unsigned maximumBandwidth;
     unsigned defaultTimeToLive;
@@ -1575,7 +1784,7 @@ class H323GatekeeperServer : public PObject
     PStringToString passwords;
 
     // Dynamic variables
-    H323EndPoint & endpoint;
+    H323EndPoint & ownerEndPoint;
     PMutex         mutex;
     time_t         identifierBase;
     unsigned       nextIdentifier;
@@ -1585,13 +1794,20 @@ class H323GatekeeperServer : public PObject
     PLIST(ListenerList, H323GatekeeperListener);
     ListenerList listeners;
 
-    PDICTIONARY(IdentifierDict, PString, H323RegisteredEndPoint);
-    PSafeDictionary<IdentifierDict, PString, H323RegisteredEndPoint> byIdentifier;
-    PStringToString byAddress;
-    PStringToString byAlias;
-    PStringToString byVoicePrefix;
+    PSafeDictionary<PString, H323RegisteredEndPoint> byIdentifier;
 
-    PSafeList<H323GatekeeperCallList, H323GatekeeperCall> activeCalls;
+    class StringMap : public PString {
+        PCLASSINFO(StringMap, PString);
+      public:
+        StringMap(const PString & from, const PString & id)
+          : PString(from), identifier(id) { }
+        PString identifier;
+    };
+    PSortedStringList byAddress;
+    PSortedStringList byAlias;
+    PSortedStringList byVoicePrefix;
+
+    PSafeSortedList<H323GatekeeperCall> activeCalls;
 
     PINDEX peakRegistrations;
     PINDEX totalRegistrations;
