@@ -27,6 +27,32 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323con.h,v $
+ * Revision 1.58  2002/11/27 06:54:52  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.57  2002/11/15 05:17:22  robertj
+ * Added facility redirect support without changing the call token for access
+ *   to the call. If it gets redirected a new H323Connection object is
+ *   created but it looks like the same thing to an application.
+ *
+ * Revision 1.56  2002/11/13 04:37:23  robertj
+ * Added ability to get (and set) Q.931 release complete cause codes.
+ *
+ * Revision 1.55  2002/11/10 06:17:26  robertj
+ * Fixed minor documentation errors.
+ *
+ * Revision 1.54  2002/11/05 00:24:09  robertj
+ * Added function to determine if Q.931 CONNECT sent/received.
+ *
+ * Revision 1.53  2002/10/31 00:31:47  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
  * Revision 1.52  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -361,6 +387,14 @@ class H323Connection : public PObject
       */
     virtual void OnCleared();
 
+    /**Determine if the call has been connected.
+       This indicates that Q.931 CONNECT has occurred. This usually means in
+       PSTN gateway environments that a charge will be made for the call. This
+       is not quite the same as IsEstablished() as that indicates the call is
+       connected AND there is media open.
+      */
+    BOOL IsConnected() const { return connectionState == HasExecutedSignalConnect || connectionState == EstablishedConnection; }
+
     /**Determine if the call has been established.
        This can be used in combination with the GetCallEndReason() function
        to determine the three main phases of a call, call setup, call
@@ -396,6 +430,8 @@ class H323Connection : public PObject
       EndedByNoEndPoint,        /// The remote party is not running an endpoint
       EndedByHostOffline,       /// The remote party host off line
       EndedByTemporaryFailure,  /// The remote failed temporarily app may retry
+      EndedByQ931Cause,         /// The remote ended the call with unmapped Q.931 cause code
+      EndedByDurationLimit,     /// Call cleared due to an enforced duration limit
       NumCallEndReasons
     };
 #if PTRACING
@@ -454,6 +490,7 @@ class H323Connection : public PObject
     /**Attach a transport to this connection as the signalling channel.
       */
     void AttachSignalChannel(
+      const PString & token,    /// New token to use to identify connection
       H323Transport * channel,  /// Transport for the PDU's
       BOOL answeringCall        /// Flag for if incoming/outgoing call.
     );
@@ -643,7 +680,7 @@ class H323Connection : public PObject
 
        The return value is TRUE if the call is to be forwarded, FALSE
        otherwise. Note that if the call is forwarded the current connection is
-       cleared with teh ended call code of EndedByCallForwarded.
+       cleared with the ended call code of EndedByCallForwarded.
       */
     virtual BOOL ForwardCall(
       const PString & forwardParty   /// Party to forward call to.
@@ -1303,6 +1340,7 @@ class H323Connection : public PObject
     );
 
     /**Select default logical channel for fast start.
+       Internal function, not for normal use.
       */
     virtual void SelectFastStartChannels(
       unsigned sessionID,   /// Session ID to find default logical channel.
@@ -1593,6 +1631,9 @@ class H323Connection : public PObject
       SendUserInputAsSeparateRFC2833,  // Not implemented
       NumSendUserInputModes
     };
+#if PTRACING
+    friend ostream & operator<<(ostream & o, SendUserInputModes m);
+#endif
 
     /**Set the user input indication transmission mode.
       */
@@ -1924,6 +1965,11 @@ class H323Connection : public PObject
      */
     BOOL HadAnsweredCall() const { return callAnswered; }
 
+    /**Get the Q.931 cause code (Q.850) that terminated this call.
+       See Q931::CauseValues for common values.
+     */
+    unsigned GetQ931Cause() const { return q931Cause; }
+
     /**Get the distinctive ring code for incoming call.
        This returns an integer from 0 to 7 that may indicate to an application
        that different ring cadences are to be used.
@@ -2056,13 +2102,22 @@ class H323Connection : public PObject
       */
     PTime GetConnectionEndTime() const { return callEndTime; }
 
-    /**Get the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio jitter delay parameter.
+       Defaults to 50ms
      */
-    WORD GetMaxAudioDelayJitter() const { return maxAudioDelayJitter; }
+    unsigned GetMinAudioJitterDelay() const { return minAudioJitterDelay; }
 
-    /**Set the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio delay jitter parameter.
+       Defaults to 250ms.
      */
-    void SetMaxAudioDelayJitter(unsigned jitter) { maxAudioDelayJitter = (WORD)jitter; }
+    unsigned GetMaxAudioJitterDelay() const { return maxAudioJitterDelay; }
+
+    /**Set the maximum audio delay jitter parameter.
+     */
+    void SetAudioJitterDelay(
+      unsigned minDelay,   // New minimum jitter buffer delay in milliseconds
+      unsigned maxDelay    // New maximum jitter buffer delay in milliseconds
+    );
 
     /**Get the UUIE PDU monitor bit mask.
      */
@@ -2100,6 +2155,14 @@ class H323Connection : public PObject
        n - n plus this connection
      */
     const int GetRemoteCallWaiting() const { return remoteCallWaiting; }
+
+    /**Set the enforced duration limit for the call.
+       This starts a timer that will automatically shut down the call when it
+       expires.
+      */
+    void SetEnforcedDurationLimit(
+      unsigned seconds  /// max duration of call in seconds
+    );
   //@}
 
 
@@ -2138,7 +2201,8 @@ class H323Connection : public PObject
     H323Capabilities   remoteCapabilities; // Capabilities remote system supports
     unsigned           remoteMaxAudioDelayJitter;
     PTimer             roundTripDelayTimer;
-    WORD               maxAudioDelayJitter;
+    unsigned           minAudioJitterDelay;
+    unsigned           maxAudioJitterDelay;
     unsigned           bandwidthAvailable;
     unsigned           uuiesRequested;
     PString            gkAccessTokenOID;
@@ -2171,6 +2235,7 @@ class H323Connection : public PObject
     PTime         connectedTime;
     PTime         callEndTime;
     CallEndReason callEndReason;
+    unsigned      q931Cause;
 
     unsigned   h225version;
     unsigned   h245version;
@@ -2188,6 +2253,7 @@ class H323Connection : public PObject
     BOOL       endSessionNeeded;
     BOOL       endSessionSent;
     PSyncPoint endSessionReceived;
+    PTimer     enforcedDurationLimit;
 
     // Used as part of a local call hold operation involving MOH
     PChannel * holdMediaChannel;
