@@ -27,6 +27,35 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323ep.h,v $
+ * Revision 1.43  2002/11/28 01:19:55  craigs
+ * Added virtual to several functions
+ *
+ * Revision 1.42  2002/11/27 06:54:52  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.41  2002/11/15 05:17:22  robertj
+ * Added facility redirect support without changing the call token for access
+ *   to the call. If it gets redirected a new H323Connection object is
+ *   created but it looks like the same thing to an application.
+ *
+ * Revision 1.40  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.39  2002/10/31 00:32:15  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.38  2002/10/23 06:06:10  robertj
+ * Added function to be smarter in using a gatekeeper for use by endpoint.
+ *
+ * Revision 1.37  2002/10/21 06:07:44  robertj
+ * Added function to set gatekeeper access token OID.
+ *
  * Revision 1.36  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -166,9 +195,11 @@
 class H225_EndpointType;
 class H225_VendorIdentifier;
 class H225_H221NonStandard;
+class H225_ServiceControlDescriptor;
 
 class H323SignalPDU;
 class H323ConnectionsCleaner;
+class H323ServiceControlSession;
 
 class OpalT120Protocol;
 class OpalT38Protocol;
@@ -198,6 +229,10 @@ class H323EndPoint : public PObject
   PCLASSINFO(H323EndPoint, PObject);
 
   public:
+    enum {
+      DefaultTcpPort = 1720
+    };
+
   /**@name Construction */
   //@{
     /**Create a new endpoint.
@@ -319,6 +354,29 @@ class H323EndPoint : public PObject
 
   /**@name Gatekeeper management */
   //@{
+    /**Use and register with an explicit gatekeeper.
+       This will call other functions according to the following table:
+
+           address    identifier   function
+           empty      empty        DiscoverGatekeeper()
+           non-empty  empty        SetGatekeeper()
+           empty      non-empty    LocateGatekeeper()
+           non-empty  non-empty    SetGatekeeperZone()
+
+       The localAddress field, if non-empty, indicates the interface on which
+       to look for the gatekeeper. An empty string is equivalent to "ip$*:*"
+       which is any interface or port.
+
+       If the endpoint is already registered with a gatekeeper that meets
+       the same criteria then the gatekeeper is not changed, otherwise it is
+       deleted (with unregistration) and new one created and registered to.
+     */
+    BOOL UseGatekeeper(
+      const PString & address = PString::Empty(),     /// Address of gatekeeper to use.
+      const PString & identifier = PString::Empty(),  /// Identifier of gatekeeper to use.
+      const PString & localAddress = PString::Empty() /// Local interface to use.
+    );
+
     /**Select and register with an explicit gatekeeper.
        This will use the specified transport and a string giving a transport
        dependent address to locate a specific gatekeeper. The endpoint will
@@ -406,13 +464,13 @@ class H323EndPoint : public PObject
 
     /**Set the H.235 password for the gatekeeper.
       */
-    void SetGatekeeperPassword(
+    virtual void SetGatekeeperPassword(
       const PString & password
     );
 
     /**Get the H.235 password for the gatekeeper.
       */
-    const PString & GetGatekeeperPassword() const { return gatekeeperPassword; }
+    virtual const PString & GetGatekeeperPassword() const { return gatekeeperPassword; }
 
     /**Create a list of authenticators for gatekeeper.
       */
@@ -776,13 +834,13 @@ class H323EndPoint : public PObject
       const PString & user                /// Username of remote endpoint
     );
 
-    /**A call back function whenever connection indicates it should be
-       forwarded.
-       Return TRUE if the call is being redirected.
+    /**A call back function when a connection indicates it is to be forwarded.
+       An H323 application may handle this call back so it can make
+       complicated decisions on if the call forward ius to take place. If it
+       decides to do so it must call MakeCall() and return TRUE.
 
-       The default behaviour simply returns FALSE and does not forward the
-       call. The H323 application needs to handle this call back so it can
-       call MakeCall() and know the token of the new H323 connection.
+       The default behaviour simply returns FALSE and that the automatic
+       call forwarding should take place. See ForwardConnection()
       */
     virtual BOOL OnConnectionForwarded(
       H323Connection & connection,    /// Connection to be forwarded
@@ -790,6 +848,20 @@ class H323EndPoint : public PObject
       const H323SignalPDU & pdu       /// Full PDU initiating forwarding
     );
 
+    /**Forward the call using the same token as the specified connection.
+       Return TRUE if the call is being redirected.
+
+       The default behaviour will replace the current call in the endpoints
+       call list using the same token as the call being redirected. Not that
+       even though the same token is being used the actual object is
+       completely mad anew.
+      */
+    virtual BOOL ForwardConnection(
+      H323Connection & connection,    /// Connection to be forwarded
+      const PString & forwardParty,   /// Remote party to forward to
+      const H323SignalPDU & pdu       /// Full PDU initiating forwarding
+    );
+
     /**A call back function whenever a connection is established.
        This indicates that a connection to a remote endpoint was established
        with a control channel and zero or more logical channels.
@@ -949,6 +1021,52 @@ class H323EndPoint : public PObject
     );
   //@}
 
+  /**@name Service Control */
+  //@{
+    /**Call back for HTTP based Service Control.
+       An application may override this to use an HTTP based channel using a
+       resource designated by the session ID. For example the session ID can
+       correspond to a browser window and the 
+
+       The default behaviour does nothing.
+      */
+    virtual void OnHTTPServiceControl(
+      unsigned operation,  /// Control operation
+      unsigned sessionId,  /// Session ID for HTTP page
+      const PString & url  /// URL to use.
+    );
+
+    /**Call back for call credit information.
+       An application may override this to display call credit information
+       on registration, or when a call is started.
+
+       The canDisplayAmountString member variable must also be set to TRUE
+       for this to operate.
+
+       The default behaviour does nothing.
+      */
+    virtual void OnCallCreditServiceControl(
+      const PString & amount,  /// UTF-8 string for amount, including currency.
+      BOOL mode          /// Flag indicating that calls will debit the account.
+    );
+
+    /**Handle incoming service control session information.
+       Default behaviour calls session.OnChange()
+     */
+    virtual void OnServiceControlSession(
+      unsigned type,
+      unsigned sessionid,
+      const H323ServiceControlSession & session,
+      H323Connection * connection
+    );
+
+    /**Create the service control session object.
+     */
+    virtual H323ServiceControlSession * CreateServiceControlSession(
+      const H225_ServiceControlDescriptor & contents
+    );
+  //@}
+
   /**@name Other services */
   //@{
     /**Create an instance of the T.120 protocol handler.
@@ -990,7 +1108,7 @@ class H323EndPoint : public PObject
        Additional aliases may be added by the use of the AddAliasName()
        function, however that list will be cleared when this function is used.
      */
-    void SetLocalUserName(
+    virtual void SetLocalUserName(
       const PString & name  /// Local name of endpoint (prime alias)
     );
 
@@ -998,7 +1116,7 @@ class H323EndPoint : public PObject
        defaults to the logged in user as obtained from the
        PProcess::GetUserName() function.
      */
-    const PString & GetLocalUserName() const { return localAliasNames[0]; }
+    virtual const PString & GetLocalUserName() const { return localAliasNames[0]; }
 
     /**Add an alias name to be used for the local end of any connections. If
        the alias name already exists in the list then is is not added again.
@@ -1056,15 +1174,39 @@ class H323EndPoint : public PObject
       BOOL mode /// New default mode
     ) { disableH245inSetup = mode; } 
 
+    /**Get the detect in-band DTMF flag.
+      */
     BOOL DetectInBandDTMFDisabled() const
       { return disableDetectInBandDTMF; }
 
-    /**Set the default H.245 tunneling mode.
+    /**Set the detect in-band DTMF flag.
       */
     void DisableDetectInBandDTMF(
       BOOL mode /// New default mode
     ) { disableDetectInBandDTMF = mode; } 
 
+    /**Get the flag indicating the endpoint can display an amount string.
+      */
+    BOOL CanDisplayAmountString() const
+      { return canDisplayAmountString; }
+
+    /**Set the flag indicating the endpoint can display an amount string.
+      */
+    void SetCanDisplayAmountString(
+      BOOL mode /// New default mode
+    ) { canDisplayAmountString = mode; } 
+
+    /**Get the flag indicating the call will automatically clear after a time.
+      */
+    BOOL CanEnforceDurationLimit() const
+      { return canEnforceDurationLimit; }
+
+    /**Set the flag indicating the call will automatically clear after a time.
+      */
+    void SetCanEnforceDurationLimit(
+      BOOL mode /// New default mode
+    ) { canEnforceDurationLimit = mode; } 
+
     /**Get Call Intrusion Protection Level of the end point.
       */
     unsigned GetCallIntrusionProtectionLevel() const { return callIntrusionProtectionLevel; }
@@ -1150,6 +1292,10 @@ class H323EndPoint : public PObject
      */
     BOOL CanAutoStartTransmitFax() const { return autoStartTransmitFax; }
 
+    /**See if should automatically do call forward of connection.
+     */
+    BOOL CanAutoCallForward() const { return autoCallForward; }
+
     /**Get the set of listeners (incoming call transports) for this endpoint.
      */
     const H323ListenerList & GetListeners() const { return listeners; }
@@ -1198,13 +1344,22 @@ class H323EndPoint : public PObject
      */
     BOOL IsMCU() const;
 
-    /**Get the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio jitter delay parameter.
+       Defaults to 50ms
      */
-    WORD GetMaxAudioDelayJitter() const { return maxAudioDelayJitter; }
+    unsigned GetMinAudioJitterDelay() const { return minAudioJitterDelay; }
 
-    /**Set the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio delay jitter parameter.
+       Defaults to 250ms.
      */
-    void SetMaxAudioDelayJitter(unsigned jitter) { maxAudioDelayJitter = (WORD)jitter; }
+    unsigned GetMaxAudioJitterDelay() const { return maxAudioJitterDelay; }
+
+    /**Set the maximum audio delay jitter parameter.
+     */
+    void SetAudioJitterDelay(
+      unsigned minDelay,   // New minimum jitter buffer delay in milliseconds
+      unsigned maxDelay    // New maximum jitter buffer delay in milliseconds
+    );
 
     /**Get the initial bandwidth parameter.
      */
@@ -1341,7 +1496,11 @@ class H323EndPoint : public PObject
 
     /**Get the iNow Gatekeeper Access Token OID.
      */
-    const PString GetGkAccessTokenOID() const { return gkAccessTokenOID; }
+    const PString & GetGkAccessTokenOID() const { return gkAccessTokenOID; }
+
+    /**Set the iNow Gatekeeper Access Token OID.
+     */
+    void SetGkAccessTokenOID(const PString & token) { gkAccessTokenOID = token; }
 
     /**Get the default timeout for Call Transfer Timer CT-T1.
      */
@@ -1411,8 +1570,8 @@ class H323EndPoint : public PObject
       unsigned capabilityLevel,     /// Intrusion capability level
       const PString & remoteParty,  /// Remote party to call
       H323Transport * transport,    /// Transport to use for call.
-      PString & token,              /// String to receive token for connection
-      void * userData = NULL        /// user data to pass to CreateConnection
+      PString & token,              /// String to use/receive token for connection
+      void * userData               /// user data to pass to CreateConnection
     );
 
     // Configuration variables, commonly changed
@@ -1425,10 +1584,13 @@ class H323EndPoint : public PObject
     BOOL        autoStartTransmitVideo;
     BOOL        autoStartReceiveFax;
     BOOL        autoStartTransmitFax;
+    BOOL        autoCallForward;
     BOOL        disableFastStart;
     BOOL        disableH245Tunneling;
     BOOL        disableH245inSetup;
     BOOL        disableDetectInBandDTMF;
+    BOOL        canDisplayAmountString;
+    BOOL        canEnforceDurationLimit;
     unsigned    callIntrusionProtectionLevel;
     H323AudioCodec::SilenceDetectionMode defaultSilenceDetection;
     H323Connection::SendUserInputModes defaultSendUserInputMode;
@@ -1438,7 +1600,8 @@ class H323EndPoint : public PObject
     WORD          manufacturerCode;
     TerminalTypes terminalType;
 
-    WORD     maxAudioDelayJitter;
+    unsigned minAudioJitterDelay;
+    unsigned maxAudioJitterDelay;
     unsigned initialBandwidth;  // in 100s of bits/sev
     BOOL     clearCallOnRoundTripFail;
 
