@@ -24,6 +24,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h450pdu.h,v $
+ * Revision 1.10  2002/11/21 22:37:24  robertj
+ * Fixed problems with unrecognized invoke APDU, thanks Andrea Bondavalli
+ *
  * Revision 1.9  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -75,6 +78,7 @@ class H323TransportAddress;
 class H323SignalPDU;
 
 class H4501_EndpointAddress;
+class H4501_InterpretationApdu;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -239,7 +243,7 @@ class H450xDispatcher : public PObject
        The default behaviour is to attempt to decode the invoke operation
        and call the corresponding OnReceived<Operation> method on the EndPoint.
      */
-    virtual BOOL OnReceivedInvoke(X880_Invoke& invoke);
+    virtual BOOL OnReceivedInvoke(X880_Invoke& invoke, H4501_InterpretationApdu& interpretation);
 
     /**Handle an incoming X880 Return Result PDU.
        The default behaviour is to attempt to match the return result
