@@ -27,6 +27,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ixjlid.h,v $
+ * Revision 1.61  2002/11/06 04:03:38  dereks
+ * Improve docs for  SetToneFilterParameters().
+ *
+ * Revision 1.60  2002/11/05 04:26:21  robertj
+ * Imported RingLine() by array from OPAL.
+ *
  * Revision 1.59  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -341,6 +347,26 @@ class OpalIxJDevice : public OpalLineInterfaceDevice
       DWORD cadence     /// Cadence bit map for ring pattern
     );
 
+    /**Begin ringing local phone set with specified cadence.
+       If nCadence is zero then stops ringing.
+
+       Note that not be possible on a given line, for example on a PSTN line
+       the ring state is determined by external hardware and cannot be
+       changed by the software.
+
+       Also note that the cadence may be ignored by particular hardware driver
+       so that only the zero or non-zero values are significant.
+
+       The ring pattern is an array of millisecond times for on and off parts
+       of the cadence. Thus the Australian ring cadence would be represented
+       by the array   unsigned AusRing[] = { 400, 200, 400, 2000 }
+      */
+    virtual BOOL RingLine(
+      unsigned line,     /// Number of line
+      PINDEX nCadence,   /// Number of entries in cadence array
+      unsigned * pattern /// Ring pattern times
+    );
+
 
     /**Determine if line has been disconnected from a call.
       */
@@ -638,6 +664,9 @@ class OpalIxJDevice : public OpalLineInterfaceDevice
     );
 
     /**Set a tones filter parameters.
+
+       The times are in centi-seconds. 
+       Thus, to have a 1 second delay, 100 is required.
       */
     virtual BOOL SetToneFilterParameters(
       unsigned line,            /// Number of line
