@@ -27,6 +27,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: lid.h,v $
+ * Revision 1.53  2002/11/05 04:26:48  robertj
+ * Imported RingLine() by array from OPAL.
+ *
+ * Revision 1.52  2002/10/29 00:46:04  dereks
+ * Fix typo in comments.
+ *
  * Revision 1.51  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -340,6 +346,26 @@ class OpalLineInterfaceDevice : public PObject
       DWORD cadence     /// Cadence bit map for ring pattern
     );
 
+    /**Begin ringing local phone set with specified cadence.
+       If nCadence is zero then stops ringing.
+
+       Note that not be possible on a given line, for example on a PSTN line
+       the ring state is determined by external hardware and cannot be
+       changed by the software.
+
+       Also note that the cadence may be ignored by particular hardware driver
+       so that only the zero or non-zero values are significant.
+
+       The ring pattern is an array of millisecond times for on and off parts
+       of the cadence. Thus the Australian ring cadence would be represented
+       by the array   unsigned AusRing[] = { 400, 200, 400, 2000 }
+      */
+    virtual BOOL RingLine(
+      unsigned line,     /// Number of line
+      PINDEX nCadence,   /// Number of entries in cadence array
+      unsigned * pattern /// Ring pattern times
+    );
+
 
     /**Determine if line has been disconnected from a call.
        This uses the hardware (and country) dependent means for determining
@@ -768,7 +794,7 @@ class OpalLineInterfaceDevice : public PObject
           1100:0.4-0.4  1100Hz with cadence 400ms on, 400ms off
           900-1300:1.5  900Hz to 1300Hz for minimum of 1.5 seconds
           425:0.4-0.2-0.4-2    425Hz with cadence
-                                400ms on, 400ms off, 400ms on, 2 seconds off
+                                400ms on, 200ms off, 400ms on, 2 seconds off
       */
     virtual BOOL SetToneFilter(
       unsigned line,              /// Number of line
