@@ -24,6 +24,12 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.h,v $
+ * Revision 1.50  2002/11/25 22:40:00  robertj
+ * Added another Q.850 code
+ *
+ * Revision 1.49  2002/11/07 03:49:49  robertj
+ * Added extra "congested" Q.931 codes.
+ *
  * Revision 1.48  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -305,37 +311,39 @@ class Q931 : public PObject
 
     enum CauseValues {
       UnknownCauseIE,
-      UnallocatedNumber         =  1,
-      NoRouteToNetwork          =  2,
-      NoRouteToDestination      =  3,
-      SendSpecialTone           =  4,
-      MisdialledTrunkPrefix     =  5,
-      ChannelUnacceptable       =  6,
-      NormalCallClearing        = 16,
-      UserBusy                  = 17,
-      NoResponse                = 18,
-      NoAnswer                  = 19,
-      SubscriberAbsent          = 20,
-      CallRejected              = 21,
-      NumberChanged             = 22,
-      Redirection               = 23,
-      ExchangeRoutingError      = 25,
-      NonSelectedUserClearing   = 26,
-      DestinationOutOfOrder     = 27,
-      InvalidNumberFormat       = 28,
-      FacilityRejected          = 29,
-      StatusEnquiryResponse     = 30,
-      NormalUnspecified         = 31,
-      NoCircuitChannelAvailable = 34,
-      NetworkOutOfOrder         = 38,
-      TemporaryFailure          = 41,
-      Congestion                = 42,
-      ResourceUnavailable       = 47,
-      InvalidCallReference      = 81,
-      IncompatibleDestination   = 88,
-      InterworkingUnspecified   = 111,
-      NonStandardReason         = 127,
-      ErrorInCauseIE            = 0x100
+      UnallocatedNumber            =  1,
+      NoRouteToNetwork             =  2,
+      NoRouteToDestination         =  3,
+      SendSpecialTone              =  4,
+      MisdialledTrunkPrefix        =  5,
+      ChannelUnacceptable          =  6,
+      NormalCallClearing           = 16,
+      UserBusy                     = 17,
+      NoResponse                   = 18,
+      NoAnswer                     = 19,
+      SubscriberAbsent             = 20,
+      CallRejected                 = 21,
+      NumberChanged                = 22,
+      Redirection                  = 23,
+      ExchangeRoutingError         = 25,
+      NonSelectedUserClearing      = 26,
+      DestinationOutOfOrder        = 27,
+      InvalidNumberFormat          = 28,
+      FacilityRejected             = 29,
+      StatusEnquiryResponse        = 30,
+      NormalUnspecified            = 31,
+      NoCircuitChannelAvailable    = 34,
+      NetworkOutOfOrder            = 38,
+      TemporaryFailure             = 41,
+      Congestion                   = 42,
+      RequestedCircuitNotAvailable = 44,
+      ResourceUnavailable          = 47,
+      InvalidCallReference         = 81,
+      ClearedRequestedCallIdentity = 86,
+      IncompatibleDestination      = 88,
+      InterworkingUnspecified      = 111,
+      NonStandardReason            = 127,
+      ErrorInCauseIE               = 0x100
     };
     void SetCause(
       CauseValues value,
