@@ -27,6 +27,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.h,v $
+ * Revision 1.43  2002/11/19 01:48:15  robertj
+ * Allowed get/set of canonical anme and tool name.
+ *
+ * Revision 1.42  2002/11/12 22:10:48  robertj
+ * Updated documentation.
+ *
+ * Revision 1.41  2002/10/31 00:33:29  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
  * Revision 1.40  2002/09/26 04:01:58  robertj
  * Fixed calculation of fraction of packets lost in RR, thanks Awais Ali
  *
@@ -442,10 +452,18 @@ class RTP_Session : public PObject
        attached to this RTP session.
       */
     void SetJitterBufferSize(
-      unsigned jitterDelay,    /// Total jitter buffer delay in milliseconds
+      unsigned minJitterDelay, /// Minimum jitter buffer delay in RTP timestamp units
+      unsigned maxJitterDelay, /// Maximum jitter buffer delay in RTP timestamp units
       PINDEX stackSize = 30000 /// Stack size for jitter thread
     );
 
+    /**Get current size of the jitter buffer.
+       This returns the currently used jitter buffer delay in RTP timestamp
+       units. It will be some value between the minimum and maximum set in
+       the SetJitterBufferSize() function.
+      */
+    unsigned GetJitterBufferSize() const;
+
     /**Read a data frame from the RTP channel.
        This function will conditionally read data from eth jitter buffer or
        directly if there is no jitter buffer enabled. An application should
@@ -559,6 +577,22 @@ class RTP_Session : public PObject
       */
     unsigned GetSessionID() const { return sessionID; }
 
+    /**Get the canonical name for the RTP session.
+      */
+    PString GetCanonicalName() const;
+
+    /**Set the canonical name for the RTP session.
+      */
+    void SetCanonicalName(const PString & name);
+
+    /**Get the tool name for the RTP session.
+      */
+    PString GetToolName() const;
+
+    /**Set the tool name for the RTP session.
+      */
+    void SetToolName(const PString & name);
+
     /**Get the user data for the session.
       */
     RTP_UserData * GetUserData() const { return userData; }
@@ -699,13 +733,20 @@ class RTP_Session : public PObject
        This is the calculated statistical variance of the interarrival
        time of received packets in milliseconds.
       */
-    DWORD GetJitterTime() const { return jitterLevel>>7; }
+    DWORD GetAvgJitterTime() const { return jitterLevel>>7; }
+
+    /**Get averaged jitter time for received packets.
+       This is the maximum value of jitterLevel for the session.
+      */
+    DWORD GetMaxJitterTime() const { return maximumJitterLevel>>7; }
   //@}
 
   protected:
     void AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
 
     unsigned           sessionID;
+    PString            canonicalName;
+    PString            toolName;
     unsigned           referenceCount;
     RTP_UserData     * userData;
     RTP_JitterBuffer * jitter;
@@ -739,6 +780,7 @@ class RTP_Session : public PObject
     DWORD maximumReceiveTime;
     DWORD minimumReceiveTime;
     DWORD jitterLevel;
+    DWORD maximumJitterLevel;
 
     unsigned txStatisticsCount;
     unsigned rxStatisticsCount;
