@@ -27,6 +27,13 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.h,v $
+ * Revision 1.38  2002/11/21 06:39:56  robertj
+ * Changed promiscuous mode to be three way. Fixes race condition in gkserver
+ *   which can cause crashes or more PDUs to be sent to the wrong place.
+ *
+ * Revision 1.37  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
  * Revision 1.36  2002/09/16 01:14:15  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -379,11 +386,11 @@ class H323Transport : public PIndirectChannel
 
   /**@name Operations */
   //@{
-    /**Get the transport dependent name of the local endpoint.
+    /**Get the transport address of the local endpoint.
       */
     virtual H323TransportAddress GetLocalAddress() const = 0;
 
-    /**Get the transport dependent name of the remote endpoint.
+    /**Get the transport address of the remote endpoint.
       */
     virtual H323TransportAddress GetRemoteAddress() const = 0;
 
@@ -437,17 +444,34 @@ class H323Transport : public PIndirectChannel
       unsigned tsap
     ) const;
 
+    /// Promiscious modes for transport
+    enum PromisciousModes {
+      AcceptFromRemoteOnly,
+      AcceptFromAnyAutoSet,
+      AcceptFromAny,
+      NumPromisciousModes
+    };
+
     /**Set read to promiscuous mode.
        Normally only reads from the specifed remote address are accepted. This
-       flag allows the remote address to be automatically set to whatever the
-       sender of the last received message was.
+       flag allows packets to be accepted from any remote, provided the
+       underlying protocol can do so. For example TCP will do nothing.
+
+       The Read() call may optionally set the remote address automatically to
+       whatever the sender host of the last received message was.
 
        Default behaviour does nothing.
       */
     virtual void SetPromiscuous(
-      BOOL promiscuous
+      PromisciousModes promiscuous
     );
 
+    /**Get the transport address of the last received PDU.
+
+       Default behaviour returns GetRemoteAddress().
+      */
+    virtual H323TransportAddress GetLastReceivedAddress() const;
+
     /**Read a protocol data unit from the transport.
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
@@ -601,18 +625,13 @@ class H323ListenerTCP : public H323Listener
   PCLASSINFO(H323ListenerTCP, H323Listener);
 
   public:
-    enum {
-      DefaultSignalPort = 1720,
-      DefaultT120Port = 1503
-    };
-
     /**Create a new listener for the TCP/IP protocol.
      */
     H323ListenerTCP(
-      H323EndPoint & endpoint,        /// Endpoint instance for channel
-      PIPSocket::Address binding = INADDR_ANY, /// Local interface to listen on
-      WORD port = DefaultSignalPort,  /// TCP port to listen for connections
-      BOOL exclusive = FALSE          /// Fail if listener port in use
+      H323EndPoint & endpoint,    /// Endpoint instance for channel
+      PIPSocket::Address binding, /// Local interface to listen on
+      WORD port,                  /// TCP port to listen for connections
+      BOOL exclusive = FALSE      /// Fail if listener port in use
     );
 
     /** Destroy the listener thread.
@@ -775,18 +794,12 @@ class H323TransportUDP : public H323TransportIP
   PCLASSINFO(H323TransportUDP, H323TransportIP);
 
   public:
-    enum {
-      MulticastRasPort = 1718,
-      DefaultRasPort = 1719
-    };
-
     /**Create a new transport channel.
      */
     H323TransportUDP(
       H323EndPoint & endpoint,                 /// H323 End Point object
       PIPSocket::Address binding = INADDR_ANY, /// Local interface to listen on
-      WORD port = 0,                           /// Local port to listen on
-      BOOL promiscuous = FALSE                 /// Initial state of flag
+      WORD port = 0                            /// Local port to listen on
     );
     ~H323TransportUDP();
 
@@ -805,15 +818,25 @@ class H323TransportUDP : public H323TransportIP
 
     /**Set read to promiscuous mode.
        Normally only reads from the specifed remote address are accepted. This
-       flag allows the remote address to be automatically set to whatever the
-       sender of the last received message was.
+       flag allows packets to be accepted from any remote, provided the
+       underlying protocol can do so.
 
-       Default behaviour does nothing.
+       The Read() call may optionally set the remote address automatically to
+       whatever the sender host of the last received message was.
+
+       Default behaviour sets the internal flag, so that Read() operates as
+       described.
       */
     virtual void SetPromiscuous(
-      BOOL promiscuous
+      PromisciousModes promiscuous
     );
 
+    /**Get the transport address of the last received PDU.
+
+       Default behaviour returns the lastReceivedAddress member variable.
+      */
+    virtual H323TransportAddress GetLastReceivedAddress() const;
+
     /**Read a protocol data unit from the transport.
        This will read using the transports mechanism for PDU boundaries, for
        example UDP is a single Read() call, while for TCP there is a TPKT
@@ -844,7 +867,8 @@ class H323TransportUDP : public H323TransportIP
     );
 
   protected:
-    BOOL promiscuousReads;
+    PromisciousModes     promiscuousReads;
+    H323TransportAddress lastReceivedAddress;
 };
 
 
