@@ -27,6 +27,24 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: channels.cxx,v $
+ * Revision 1.135  2002/12/18 11:20:49  craigs
+ * Fixed problem with T.38 channels SEGVing thanks to Vyacheslav Frolov
+ *
+ * Revision 1.134  2002/12/17 08:48:09  robertj
+ * Set silence suppression mode earlier in codec life so gets correct
+ *   value for silenceSuppression in fast start OLC's.
+ *
+ * Revision 1.133  2002/12/16 08:20:04  robertj
+ * Fixed problem where a spurious RTP packet full of zeros could be sent
+ *   at the beginning of the transmission, thanks Bruce Fitzsimons
+ *
+ * Revision 1.132  2002/11/26 02:59:25  robertj
+ * Added logging to help find logical channel thread stop failures.
+ *
+ * Revision 1.131  2002/10/31 00:37:47  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
  * Revision 1.130  2002/08/05 10:03:47  robertj
  * Cosmetic changes to normalise the usage of pragma interface/implementation.
  *
@@ -516,6 +534,7 @@ H323LogicalChannelThread::H323LogicalChannelThread(H323EndPoint & endpoint,
             rx ? "LogChanRx:%0x" : "LogChanTx:%0x"),
     channel(c)
 {
+  PTRACE(4, "LogChan\tStarting logical channel thread " << this);
   receiver = rx;
   Resume();
 }
@@ -523,6 +542,7 @@ H323LogicalChannelThread::H323LogicalChannelThread(H323EndPoint & endpoint,
 
 void H323LogicalChannelThread::Main()
 {
+  PTRACE(4, "LogChan\tStarted logical channel thread " << this);
   if (receiver)
     channel.Receive();
   else
@@ -644,6 +664,7 @@ void H323Channel::CleanUpOnTermination()
 
   // If we have a receiver thread, wait for it to die.
   if (receiveThread != NULL) {
+    PTRACE(4, "LogChan\tAwaiting termination of " << receiveThread << ' ' << receiveThread->GetThreadName());
     PAssert(receiveThread->WaitForTermination(10000), "Receive media thread did not terminate");
     delete receiveThread;
     receiveThread = NULL;
@@ -651,6 +672,7 @@ void H323Channel::CleanUpOnTermination()
 
   // If we have a transmitter thread, wait for it to die.
   if (transmitThread != NULL) {
+    PTRACE(4, "LogChan\tAwaiting termination of " << transmitThread << ' ' << transmitThread->GetThreadName());
     PAssert(transmitThread->WaitForTermination(10000), "Transmit media thread did not terminate");
     delete transmitThread;
     transmitThread = NULL;
@@ -779,9 +801,12 @@ BOOL H323Channel::Open()
 
 H323Codec * H323Channel::GetCodec() const
 {
-  if (codec == NULL)
+  if (codec == NULL) {
     ((H323Channel*)this)->codec = capability->CreateCodec(
                   GetDirection() == IsReceiver ? H323Codec::Decoder : H323Codec::Encoder);
+    if (codec && codec->IsDescendant(H323AudioCodec::Class()))
+      ((H323AudioCodec*)codec)->SetSilenceDetectionMode(endpoint.GetSilenceDetectionMode()); 
+  }
 
   return codec;
 }
@@ -1195,6 +1220,8 @@ void H323_RTPChannel::Transmit()
   unsigned frameOffset = 0;
   unsigned frameCount = 0;
   DWORD rtpTimestamp = 0;
+  frame.SetPayloadSize(0);
+
 #if PTRACING
   DWORD lastDisplayedTimestamp = 0;
   CodecReadAnalyser * codecReadAnalysis = NULL;
@@ -1331,7 +1358,8 @@ void H323_RTPChannel::Receive()
 
   // if jitter buffer required, start the thread that is on the other end of it
   if (mediaFormat.NeedsJitterBuffer())
-    rtpSession.SetJitterBufferSize(connection.GetMaxAudioDelayJitter()*mediaFormat.GetTimeUnits(),
+    rtpSession.SetJitterBufferSize(connection.GetMinAudioJitterDelay()*mediaFormat.GetTimeUnits(),
+                                   connection.GetMaxAudioJitterDelay()*mediaFormat.GetTimeUnits(),
                                    endpoint.GetJitterThreadStackSize());
 
   // Keep time using th RTP timestamps.
