@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: codecs.cxx,v $
+ * Revision 1.80  2002/12/16 09:11:19  robertj
+ * Added new video bit rate control, thanks Walter H. Whitlock
+ *
  * Revision 1.79  2002/08/05 10:03:47  robertj
  * Cosmetic changes to normalise the usage of pragma interface/implementation.
  *
@@ -462,9 +465,12 @@ H323VideoCodec::H323VideoCodec(const char * fmt, Direction dir)
   : H323Codec(fmt, dir)
 {
   frameWidth = frameHeight = 0;
-  bytesSent = 0;
-  videoBitRateControlModes = DynamicVideoQuality;
-  startTime = PTimer::Tick();
+  targetFrameTimeMs = 0;
+  videoBitRateControlModes = None;
+
+  oldLength = 0;
+  oldTime = 0;
+  newTime = 0;
 }
 
 
@@ -612,29 +618,27 @@ void H323VideoCodec::Close()
 }
 
 
-BOOL H323VideoCodec::SetAverageBitRate(unsigned bitRate, unsigned modes)
+BOOL H323VideoCodec::SetMaxBitRate(unsigned bitRate)
 {
-  PTRACE(1,"SetAverageBandwidth for video to " << bitRate << "bps");
+  PTRACE(1,"Set bitRateHighLimit for video to " << bitRate << " bps");
         
-  if (bitRate == 0) {       // disable constant bitrate
-    frameRateDivider = 1;
-    videoBitRate = bitRate;
-    return TRUE;    
-  }
+  bitRateHighLimit = bitRate;
 
-  if (bitRate < 1024)
-    bitRate = 1024;      // from 1kbit/s
-  else if (bitRate > 2048*1024)
-    bitRate = 2048*1024;  // to 2Mbit/s
-
-  videoBitRate    = bitRate;
-  defBitsPerFrame = (int)((float)videoBitRate / estFps);
-  reqBitsPerFrame = defBitsPerFrame*frameRateDivider;
-
-  videoBitRateControlModes = modes;
+  if (0 == bitRateHighLimit) // disable bitrate control
+    videoBitRateControlModes &= ~AdaptivePacketDelay;
   return TRUE;
 }
 
+BOOL H323VideoCodec::SetTargetFrameTimeMs(unsigned ms)
+{
+  PTRACE(1,"Set targetFrameTimeMs for video to " << ms << " milliseconds");
+
+  targetFrameTimeMs = ms;
+
+  if (0 == targetFrameTimeMs)
+    videoBitRateControlModes &= ~DynamicVideoQuality;
+  return TRUE;
+ }
 
 void H323VideoCodec::SendMiscCommand(unsigned command)
 {
