@@ -27,6 +27,45 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323ep.cxx,v $
+ * Revision 1.149  2003/01/06 06:13:37  robertj
+ * Increased maximum possible jitter configuration to 10 seconds.
+ *
+ * Revision 1.148  2002/11/27 06:54:57  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.147  2002/11/19 07:07:44  robertj
+ * Changed priority so H.235 standard authentication used by preference.
+ *
+ * Revision 1.146  2002/11/15 06:53:24  robertj
+ * Fixed non facility redirect calls being able to be cleared!
+ *
+ * Revision 1.145  2002/11/15 05:17:26  robertj
+ * Added facility redirect support without changing the call token for access
+ *   to the call. If it gets redirected a new H323Connection object is
+ *   created but it looks like the same thing to an application.
+ *
+ * Revision 1.144  2002/11/14 22:06:08  robertj
+ * Increased default maximum number of ports.
+ *
+ * Revision 1.143  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.142  2002/10/31 00:42:41  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.141  2002/10/24 07:18:24  robertj
+ * Changed gatekeeper call so if can do local DNS lookup or using IP address
+ *   then uses destCallSignalAddress for ARQ instead of destinationInfo.
+ *
+ * Revision 1.140  2002/10/23 06:06:13  robertj
+ * Added function to be smarter in using a gatekeeper for use by endpoint.
+ *
  * Revision 1.139  2002/10/01 06:38:36  robertj
  * Removed GNU compiler warning
  *
@@ -700,16 +739,20 @@ H323EndPoint::H323EndPoint()
 
   autoStartReceiveVideo = autoStartTransmitVideo = TRUE;
   autoStartReceiveFax = autoStartTransmitFax = FALSE;
+  autoCallForward = TRUE;
   disableFastStart = FALSE;
   disableH245Tunneling = FALSE;
   disableH245inSetup = FALSE;
   disableDetectInBandDTMF = FALSE;
+  canDisplayAmountString = FALSE;
+  canEnforceDurationLimit = TRUE;
   callIntrusionProtectionLevel = 3; //H45011_CIProtectionLevel::e_fullProtection;
   defaultSilenceDetection = H323AudioCodec::AdaptiveSilenceDetection;
   defaultSendUserInputMode = H323Connection::SendUserInputAsString;
 
   terminalType = e_TerminalOnly;
-  maxAudioDelayJitter = 50; // milliseconds
+  minAudioJitterDelay = 50;  // milliseconds
+  maxAudioJitterDelay = 250; // milliseconds
   initialBandwidth = 100000; // Standard 10base LAN in 100's of bits/sec
   clearCallOnRoundTripFail = FALSE;
 
@@ -718,7 +761,7 @@ H323EndPoint::H323EndPoint()
   manufacturerCode = 61; // Allocated by Australian Communications Authority, Oct 2000
 
   rtpIpPorts.current = rtpIpPorts.base = 5000;
-  rtpIpPorts.max = 5199;
+  rtpIpPorts.max = 5999;
 
   // use dynamic port allocation by default
   tcpPorts.current = tcpPorts.base = tcpPorts.max = 0;
@@ -895,6 +938,43 @@ void H323EndPoint::ReorderCapabilities(const PStringArray & preferenceOrder)
 }
 
 
+BOOL H323EndPoint::UseGatekeeper(const PString & address,
+                                 const PString & identifier,
+                                 const PString & localAddress)
+{
+  if (gatekeeper != NULL) {
+    BOOL same = TRUE;
+
+    if (!address)
+      same = gatekeeper->GetTransport().GetRemoteAddress().IsEquivalent(address);
+
+    if (!same && !identifier)
+      same = gatekeeper->GetIdentifier() == identifier;
+
+    if (!same && !localAddress)
+      same = gatekeeper->GetTransport().GetLocalAddress().IsEquivalent(localAddress);
+
+    if (same) {
+      PTRACE(2, "H323\tUsing existing gatekeeper " << *gatekeeper);
+      return TRUE;
+    }
+  }
+
+  if (address.IsEmpty()) {
+    if (identifier.IsEmpty())
+      return DiscoverGatekeeper();
+    else
+      return LocateGatekeeper(identifier);
+  }
+  else {
+    if (identifier.IsEmpty())
+      return SetGatekeeper(address);
+    else
+      return SetGatekeeperZone(address, identifier);
+  }
+}
+
+
 BOOL H323EndPoint::SetGatekeeper(const PString & address, H323Transport * transport)
 {
   H323Gatekeeper * gk = InternalCreateGatekeeper(transport);
@@ -1000,10 +1080,10 @@ H235Authenticators H323EndPoint::CreateAuthenticators()
 {
   H235Authenticators authenticators;
 
-  authenticators.Append(new H235AuthSimpleMD5);
 #if P_SSL
   authenticators.Append(new H235AuthProcedure1);
 #endif
+  authenticators.Append(new H235AuthSimpleMD5);
 
   return authenticators;
 }
@@ -1044,7 +1124,7 @@ BOOL H323EndPoint::StartListener(const H323TransportAddress & iface)
   H323Listener * listener;
 
   if (iface.IsEmpty())
-    listener = new H323ListenerTCP(*this);
+    listener = new H323ListenerTCP(*this, INADDR_ANY, DefaultTcpPort);
   else
     listener = iface.CreateListener(*this);
 
@@ -1111,9 +1191,14 @@ H323Connection * H323EndPoint::MakeCall(const PString & remoteParty,
                                         PString & token,
                                         void * userData)
 {
-  H323Connection * connection = InternalMakeCall(PString(), PString(), UINT_MAX,
-                                                 remoteParty, transport,
-                                                 token, userData);
+  token = PString::Empty();
+  H323Connection * connection = InternalMakeCall(PString::Empty(),
+                                                 PString::Empty(),
+                                                 UINT_MAX,
+                                                 remoteParty,
+                                                 transport,
+                                                 token,
+                                                 userData);
   if (connection != NULL)
     connection->Unlock();
   return connection;
@@ -1125,12 +1210,18 @@ H323Connection * H323EndPoint::MakeCallLocked(const PString & remoteParty,
                                               void * userData,
                                               H323Transport * transport)
 {
-  return InternalMakeCall(PString(), PString(), UINT_MAX,
-                          remoteParty, transport, token, userData);
+  token = PString::Empty();
+  return InternalMakeCall(PString::Empty(),
+                          PString::Empty(),
+                          UINT_MAX,
+                          remoteParty,
+                          transport,
+                          token,
+                          userData);
 }
 
 
-H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
+H323Connection * H323EndPoint::InternalMakeCall(const PString & trasferFromToken,
                                                 const PString & callIdentity,
                                                 unsigned capabilityLevel,
                                                 const PString & remoteParty,
@@ -1157,15 +1248,33 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
     }
   }
 
+  H323Connection * connection;
+
   connectionsMutex.Wait();
 
   unsigned lastReference;
-  do {
-    lastReference = Q931::GenerateCallReference();
-    newToken = BuildConnectionToken(*transport, lastReference, FALSE);
-  } while (connectionsActive.Contains(newToken));
+  if (newToken.IsEmpty()) {
+    do {
+      lastReference = Q931::GenerateCallReference();
+      newToken = BuildConnectionToken(*transport, lastReference, FALSE);
+    } while (connectionsActive.Contains(newToken));
+  }
+  else {
+    lastReference = newToken.Mid(newToken.Find('/')+1).AsUnsigned();
 
-  H323Connection * connection = CreateConnection(lastReference, userData, transport, NULL);
+    // Move old connection on token to new value and flag for removal
+    PString adjustedToken;
+    unsigned tieBreaker = 0;
+    do {
+      adjustedToken = newToken + "-replaced";
+      adjustedToken.sprintf("-%u", ++tieBreaker);
+    } while (connectionsActive.Contains(adjustedToken));
+    connectionsActive.SetAt(adjustedToken, connectionsActive.RemoveAt(newToken));
+    connectionsToBeCleaned += adjustedToken;
+    PTRACE(3, "H323\tOverwriting call " << newToken << ", renamed to " << adjustedToken);
+  }
+
+  connection = CreateConnection(lastReference, userData, transport, NULL);
   if (connection == NULL) {
     PTRACE(1, "H323\tCreateConnection returned NULL");
     connectionsMutex.Signal();
@@ -1178,11 +1287,12 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
 
   connectionsMutex.Signal();
 
-  connection->AttachSignalChannel(transport, FALSE);
+  connection->AttachSignalChannel(newToken, transport, FALSE);
+
   if (capabilityLevel == UINT_MAX)
-    connection->HandleTransferCall(existingToken, callIdentity);
+    connection->HandleTransferCall(trasferFromToken, callIdentity);
   else {
-    connection->HandleIntrudeCall(existingToken, callIdentity);
+    connection->HandleIntrudeCall(trasferFromToken, callIdentity);
     connection->IntrudeCall(capabilityLevel);
   }
 
@@ -1205,15 +1315,26 @@ void H323EndPoint::ParsePartyName(const PString & remoteParty,
   if (at != P_MAX_INDEX) {
     alias = remoteParty.Left(at++);
     address = remoteParty.Mid(at);
+    return;
   }
-  else if (gatekeeper != NULL) {
-    alias = remoteParty;
-    address = H323TransportAddress();
-  }
-  else {
-    alias = "";
+
+  if (gatekeeper == NULL) {
+    alias = PString::Empty();
     address = remoteParty;
+    return;
   }
+
+  if (remoteParty.FindOneOf("$.:") != P_MAX_INDEX) {
+    address = remoteParty;
+    PIPSocket::Address ip;
+    if (address.GetIpAddress(ip) && ip.IsValid()) {
+      alias = PString::Empty();
+      return;
+    }
+  }
+
+  alias = remoteParty;
+  address = PString::Empty();
 }
 
 
@@ -1223,9 +1344,14 @@ H323Connection * H323EndPoint::SetupTransfer(const PString & oldToken,
                                              PString & newToken,
                                              void * userData)
 {
-  H323Connection* connection = InternalMakeCall(oldToken, callIdentity, UINT_MAX,
-                                                remoteParty, NULL,
-                                                newToken, userData);
+  newToken = PString::Empty();
+  H323Connection* connection = InternalMakeCall(oldToken,
+                                                callIdentity,
+                                                UINT_MAX,
+                                                remoteParty,
+                                                NULL,
+                                                newToken,
+                                                userData);
   if (connection != NULL)
      connection->Unlock();
    return connection;
@@ -1280,9 +1406,14 @@ H323Connection * H323EndPoint::IntrudeCall(const PString & remoteParty,
                                         unsigned capabilityLevel,
                                         void * userData)
 {
-  H323Connection * connection = InternalMakeCall(PString(), PString(), capabilityLevel,
-                                                 remoteParty, transport,
-                                                 token, userData);
+  token = PString::Empty();
+  H323Connection * connection = InternalMakeCall(PString::Empty(),
+                                                 PString::Empty(),
+                                                 capabilityLevel,
+                                                 remoteParty,
+                                                 transport,
+                                                 token,
+                                                 userData);
   if (connection != NULL)
     connection->Unlock();
   return connection;
@@ -1566,6 +1697,28 @@ BOOL H323EndPoint::OnConnectionForwarded(H323Connection & /*connection*/,
 }
 
 
+BOOL H323EndPoint::ForwardConnection(H323Connection & connection,
+                                     const PString & forwardParty,
+                                     const H323SignalPDU & /*pdu*/)
+{
+  PString token = connection.GetCallToken();
+  H323Connection * newConnection = InternalMakeCall(PString::Empty(),
+                                                    PString::Empty(),
+                                                    UINT_MAX,
+                                                    forwardParty,
+                                                    NULL,
+                                                    token,
+                                                    NULL);
+  if (newConnection == NULL)
+    return FALSE;
+
+  connection.SetCallEndReason(H323Connection::EndedByCallForwarded);
+
+  newConnection->Unlock();
+  return TRUE;
+}
+
+
 void H323EndPoint::OnConnectionEstablished(H323Connection & /*connection*/,
                                            const PString & /*token*/)
 {
@@ -1631,7 +1784,7 @@ H323Connection * H323EndPoint::OnIncomingConnection(H323Transport * transport,
     connectionsMutex.Signal();
   }
 
-  connection->AttachSignalChannel(transport, TRUE);
+  connection->AttachSignalChannel(token, transport, TRUE);
 
   return connection;
 }
@@ -1772,6 +1925,41 @@ void H323EndPoint::OnUserInputTone(H323Connection & connection,
 }
 
 
+void H323EndPoint::OnHTTPServiceControl(unsigned /*opeartion*/,
+                                        unsigned /*sessionId*/,
+                                        const PString & /*url*/)
+{
+}
+
+
+void H323EndPoint::OnCallCreditServiceControl(const PString & /*amount*/, BOOL /*mode*/)
+{
+}
+
+
+void H323EndPoint::OnServiceControlSession(unsigned type,
+                                           unsigned sessionId,
+                                           const H323ServiceControlSession & session,
+                                           H323Connection * connection)
+{
+  session.OnChange(type, sessionId, *this, connection);
+}
+
+
+H323ServiceControlSession * H323EndPoint::CreateServiceControlSession(const H225_ServiceControlDescriptor & contents)
+{
+  switch (contents.GetTag()) {
+    case H225_ServiceControlDescriptor::e_url :
+      return new H323HTTPServiceControl(contents);
+
+    case H225_ServiceControlDescriptor::e_callCreditServiceControl :
+      return new H323CallCreditServiceControl(contents);
+  }
+
+  return NULL;
+}
+
+
 OpalT120Protocol * H323EndPoint::CreateT120ProtocolHandler(const H323Connection &) const
 {
   return NULL;
@@ -1908,6 +2096,20 @@ BOOL H323EndPoint::IsMCU() const
 }
 
 
+void H323EndPoint::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
+{
+  PAssert(minDelay <= 10000 && maxDelay <= 10000, PInvalidParameter);
+
+  if (minDelay < 10)
+    minDelay = 10;
+  minAudioJitterDelay = minDelay;
+
+  if (maxDelay < minDelay)
+    maxDelay = minDelay;
+  maxAudioJitterDelay = maxDelay;
+}
+
+
 void H323EndPoint::PortInfo::Set(unsigned newBase,
                                  unsigned newMax,
                                  unsigned range,
@@ -1958,7 +2160,7 @@ WORD H323EndPoint::PortInfo::GetNext(unsigned increment)
 
 void H323EndPoint::SetTCPPorts(unsigned tcpBase, unsigned tcpMax)
 {
-  tcpPorts.Set(tcpBase, tcpMax, 49, 0);
+  tcpPorts.Set(tcpBase, tcpMax, 99, 0);
 }
 
 
@@ -1970,7 +2172,7 @@ WORD H323EndPoint::GetNextTCPPort()
 
 void H323EndPoint::SetUDPPorts(unsigned udpBase, unsigned udpMax)
 {
-  udpPorts.Set(udpBase, udpMax, 99, 0);
+  udpPorts.Set(udpBase, udpMax, 199, 0);
 }
 
 
@@ -1982,7 +2184,7 @@ WORD H323EndPoint::GetNextUDPPort()
 
 void H323EndPoint::SetRtpIpPorts(unsigned rtpIpBase, unsigned rtpIpMax)
 {
-  rtpIpPorts.Set((rtpIpBase+1)&0xfffe, rtpIpMax&0xfffe, 199, 5000);
+  rtpIpPorts.Set((rtpIpBase+1)&0xfffe, rtpIpMax&0xfffe, 999, 5000);
 }
 
 
