@@ -27,6 +27,18 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: ixjunix.cxx,v $
+ * Revision 1.134  2002/11/05 04:44:20  robertj
+ * Fixed typo
+ *
+ * Revision 1.133  2002/11/05 04:38:07  robertj
+ * Changed IsLineDisconnected() to work with POTSLine
+ *
+ * Revision 1.132  2002/11/05 04:27:34  robertj
+ * Imported RingLine() by array from OPAL.
+ *
+ * Revision 1.131  2002/10/24 21:06:28  dereks
+ * Add additional PTRACE statements to aid in debugging.
+ *
  * Revision 1.130  2002/10/01 06:43:01  robertj
  * Removed GNU compiler warning
  *
@@ -1008,7 +1020,7 @@ BOOL OpalIxJDevice::IsLineRinging(unsigned line, DWORD * /*cadence*/)
 
 BOOL OpalIxJDevice::RingLine(unsigned line, DWORD cadence)
 {
-  if (line != 0)
+  if (line != POTSLine)
     return FALSE;
 
   if (cadence == 0)
@@ -1032,11 +1044,25 @@ BOOL OpalIxJDevice::RingLine(unsigned line, DWORD cadence)
 }
 
 
+BOOL OpalIxJDevice::RingLine(unsigned line, PINDEX nCadence, unsigned * pattern)
+{
+  if (line >= GetLineCount())
+    return FALSE;
+
+  if (line != POTSLine)
+    return FALSE;
+
+  return RingLine(line, nCadence != 0 ? 0xaaa : 0);
+}
+
+
 BOOL OpalIxJDevice::IsLineDisconnected(unsigned line, BOOL checkForWink)
 {
+  if (line >= GetLineCount())
+    return FALSE;
 
   if (line != PSTNLine)
-    return FALSE;
+    return !IsLineOffHook(line);
 
   if (checkForWink) {
 
@@ -1328,6 +1354,7 @@ BOOL OpalIxJDevice::SetRawCodec(unsigned line)
 
   if (!SetReadFormat (line, CodecInfo[0].mediaFormat) ||
       !SetWriteFormat(line, CodecInfo[0].mediaFormat)) {
+    PTRACE(1, "IXJ\t Failed to set raw codec");
     StopReadCodec(line);
     StopWriteCodec(line);
     return FALSE;
@@ -1424,8 +1451,10 @@ BOOL OpalIxJDevice::ReadFrame(unsigned, void * buffer, PINDEX & count)
 
   count = 0;
 
-  if (readStopped) 
-    return FALSE;
+  if (readStopped) {
+      PTRACE(1, "IXJ\tRead stopped, so ReadFrame returns false");    
+      return FALSE;
+  }
 
   if (writeStopped) {
     PThread::Current()->Sleep(30);
@@ -1667,6 +1696,9 @@ unsigned OpalIxJDevice::GetAverageSignalLevel(unsigned, BOOL playback)
 
 BOOL OpalIxJDevice::EnableAudio(unsigned line, BOOL enable)
 {
+  if (line >= GetLineCount())
+    return FALSE;
+
   int port = PORT_SPEAKER;
 
   if (enable) {
