@@ -24,6 +24,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: lid.cxx,v $
+ * Revision 1.90  2002/12/02 03:06:26  robertj
+ * Fixed over zealous removal of code when NO_AUDIO_CODECS set.
+ *
+ * Revision 1.89  2002/11/05 04:27:12  robertj
+ * Imported RingLine() by array from OPAL.
+ *
+ * Revision 1.88  2002/10/30 05:54:17  craigs
+ * Fixed compatibilty problems with G.723.1 6k3 and 5k3
+ *
  * Revision 1.87  2002/08/05 10:03:48  robertj
  * Cosmetic changes to normalise the usage of pragma interface/implementation.
  *
@@ -424,6 +433,12 @@ BOOL OpalLineInterfaceDevice::RingLine(unsigned, DWORD)
 }
 
 
+BOOL OpalLineInterfaceDevice::RingLine(unsigned, PINDEX, unsigned *)
+{
+  return FALSE;
+}
+
+
 BOOL OpalLineInterfaceDevice::IsLineDisconnected(unsigned line, BOOL)
 {
   return IsToneDetected(line) == BusyTone;
@@ -1247,9 +1262,9 @@ static struct {
   // Do not alter the order of the next four entries with understanding what
   // H323_LIDCapability::OnReceivedPDU() does with them!
 #define G7231A_63_INDEX 1
-  { OPAL_G7231A_5k3,   20,   8,  3, TRUE,  H245_AudioCapability::e_g7231 },
+  { OPAL_G7231A_5k3,   24,   8,  3, TRUE,  H245_AudioCapability::e_g7231 },
   { OPAL_G7231A_6k3,   24,   8,  3, TRUE,  H245_AudioCapability::e_g7231 },
-  { OPAL_G7231_5k3,    20,   8,  3, FALSE, H245_AudioCapability::e_g7231 },
+  { OPAL_G7231_5k3,    24,   8,  3, FALSE, H245_AudioCapability::e_g7231 },
   { OPAL_G7231_6k3,    24,   8,  3, FALSE, H245_AudioCapability::e_g7231 },
 
   { OPAL_G729A,        10,  24,  6, FALSE, H245_AudioCapability::e_g729AnnexA },
@@ -1261,8 +1276,6 @@ static struct {
 };
 
 
-#ifndef NO_H323_AUDIO_CODECS
-
 H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_G711_ALaw_Capability, OPAL_G711_ALAW_64K"{hw}", H323_NO_EP_VAR)
 {
   return new H323_LIDCapability(OpalG711ALaw);
@@ -1316,8 +1329,6 @@ H323_REGISTER_CAPABILITY_FUNCTION(H323_LID_G7231A_5k3_Capability, OPAL_G7231A_5k
 #define G7231_CISCO OPAL_G7231A_6k3"-Cisco{hw}"
 H323_REGISTER_CAPABILITY(H323_CiscoG7231aLIDCapability, G7231_CISCO);
 
-#endif // NO_H323_AUDIO_CODECS
-
 
 /////////////////////////////////////////////////////////////////////////////
 
