@@ -27,6 +27,9 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: q931.cxx,v $
+ * Revision 1.55  2002/11/19 06:19:25  robertj
+ * Added extra "congested" Q.931 codes.
+ *
  * Revision 1.54  2002/08/06 02:27:39  robertj
  * GNU C++ v3 compatibility.
  *
@@ -257,6 +260,7 @@ static POrdinalToString::Initialiser CauseNames[] = {
   { Q931::NetworkOutOfOrder,         "Network out of order"         },
   { Q931::TemporaryFailure,          "Temporary failure"            },
   { Q931::Congestion,                "Congestion"                   },
+  { Q931::RequestedCircuitNotAvailable,"RequestedCircuitNotAvailable" },
   { Q931::ResourceUnavailable,       "Resource unavailable"         },
   { Q931::InvalidCallReference,      "Invalid call reference"       },
   { Q931::IncompatibleDestination,   "Incompatible destination"     },
