@@ -27,6 +27,20 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: rtp.cxx,v $
+ * Revision 1.83  2002/11/19 01:48:00  robertj
+ * Allowed get/set of canonical anme and tool name.
+ *
+ * Revision 1.82  2002/11/05 03:32:04  robertj
+ * Added session ID to trace logs.
+ * Fixed possible extra wait exiting RTP read loop on close.
+ *
+ * Revision 1.81  2002/11/05 01:55:50  robertj
+ * Added comments about strange mutex usage.
+ *
+ * Revision 1.80  2002/10/31 00:47:07  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
  * Revision 1.79  2002/09/26 04:01:49  robertj
  * Fixed calculation of fraction of packets lost in RR, thanks Awais Ali
  *
@@ -548,7 +562,9 @@ void RTP_UserData::OnRxStatistics(const RTP_Session & /*session*/) const
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_Session::RTP_Session(unsigned id, RTP_UserData * data)
-  : reportTimeInterval(0, 12),  // Seconds
+  : canonicalName(PProcess::Current().GetUserName()),
+    toolName(PProcess::Current().GetName()),
+    reportTimeInterval(0, 12),  // Seconds
     reportTimer(reportTimeInterval)
 {
   PAssert(id > 0 && id < 256, PInvalidParameter);
@@ -582,6 +598,7 @@ RTP_Session::RTP_Session(unsigned id, RTP_UserData * data)
   maximumReceiveTime = 0;
   minimumReceiveTime = 0;
   jitterLevel = 0;
+  maximumJitterLevel = 0;
 
   txStatisticsCount = 0;
   rxStatisticsCount = 0;
@@ -613,13 +630,46 @@ RTP_Session::~RTP_Session()
             "    averageReceiveTime= " << averageReceiveTime << "\n"
             "    maximumReceiveTime= " << maximumReceiveTime << "\n"
             "    minimumReceiveTime= " << minimumReceiveTime << "\n"
-            "    jitter            = " << (jitterLevel >> 7)
+            "    averageJitter     = " << (jitterLevel >> 7) << "\n"
+            "    maximumJitter     = " << (maximumJitterLevel >> 7)
             );
   delete userData;
   delete jitter;
 }
 
 
+PString RTP_Session::GetCanonicalName() const
+{
+  PWaitAndSignal mutex(reportMutex);
+  PString s = canonicalName;
+  s.MakeUnique();
+  return s;
+}
+
+
+void RTP_Session::SetCanonicalName(const PString & name)
+{
+  PWaitAndSignal mutex(reportMutex);
+  canonicalName = name;
+}
+
+
+PString RTP_Session::GetToolName() const
+{
+  PWaitAndSignal mutex(reportMutex);
+  PString s = toolName;
+  s.MakeUnique();
+  return s;
+}
+
+
+void RTP_Session::SetToolName(const PString & name)
+{
+  PWaitAndSignal mutex(reportMutex);
+  toolName = name;
+}
+
+
 void RTP_Session::SetUserData(RTP_UserData * data)
 {
   delete userData;
@@ -627,21 +677,29 @@ void RTP_Session::SetUserData(RTP_UserData * data)
 }
 
 
-void RTP_Session::SetJitterBufferSize(unsigned jitterDelay, PINDEX stackSize)
+void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
+                                      unsigned maxJitterDelay,
+                                      PINDEX stackSize)
 {
-  if (jitterDelay == 0) {
+  if (minJitterDelay == 0 && maxJitterDelay == 0) {
     delete jitter;
     jitter = NULL;
   }
   else if (jitter != NULL)
-    jitter->SetDelay(jitterDelay);
+    jitter->SetDelay(minJitterDelay, maxJitterDelay);
   else {
     SetIgnoreOutOfOrderPackets(FALSE);
-    jitter = new RTP_JitterBuffer(*this, jitterDelay, stackSize);
+    jitter = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, stackSize);
   }
 }
 
 
+unsigned RTP_Session::GetJitterBufferSize() const
+{
+  return jitter != NULL ? jitter->GetJitterTime() : 0;
+}
+
+
 BOOL RTP_Session::ReadBufferedData(DWORD timestamp, RTP_DataFrame & frame)
 {
   if (jitter != NULL)
@@ -822,6 +880,8 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveData(const RTP_DataFrame &
         if (variance < 0)
           variance = -variance;
         jitterLevel += variance - ((jitterLevel+8) >> 4);
+        if (jitterLevel > maximumJitterLevel)
+          maximumJitterLevel = jitterLevel;
       }
     }
     else if (sequenceNumber < expectedSequenceNumber) {
@@ -886,7 +946,8 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveData(const RTP_DataFrame &
             " avgTime=" << averageReceiveTime <<
             " maxTime=" << maximumReceiveTime <<
             " minTime=" << minimumReceiveTime <<
-            " jitter=" << (jitterLevel >> 7)
+            " jitter=" << (jitterLevel >> 7) <<
+            " maxJitter=" << (maximumJitterLevel >> 7)
             );
 
   if (userData != NULL)
@@ -952,13 +1013,12 @@ BOOL RTP_Session::SendReport()
   }
 
   // Add the SDES part to compound RTCP packet
-  PString description = PProcess::Current().GetUserName() + '@' + GetLocalHostName();
-  PTRACE(2, "RTP\tSending SDES: " << description);
+  PTRACE(2, "RTP\tSending SDES: " << canonicalName);
   report.WriteNextCompound();
 
   RTP_ControlFrame::SourceDescription & sdes = report.AddSourceDescription(syncSourceOut);
-  report.AddSourceDescriptionItem(sdes, RTP_ControlFrame::e_CNAME, description);
-  report.AddSourceDescriptionItem(sdes, RTP_ControlFrame::e_TOOL, PProcess::Current().GetName());
+  report.AddSourceDescriptionItem(sdes, RTP_ControlFrame::e_CNAME, canonicalName);
+  report.AddSourceDescriptionItem(sdes, RTP_ControlFrame::e_TOOL, toolName);
 
   // Wait a fuzzy amount of time so things don't get into lock step
   int interval = (int)reportTimeInterval.GetMilliSeconds();
@@ -1209,7 +1269,7 @@ RTP_Session * RTP_SessionManager::UseSession(unsigned sessionID)
 
   RTP_Session * session = sessions.GetAt(sessionID);
   if (session == NULL)
-    return NULL;
+    return NULL;  // Deliberately have not release mutex here! See AddSession.
 
   PTRACE(3, "RTP\tFound existing session " << sessionID);
   session->IncrementReference();
@@ -1225,6 +1285,8 @@ void RTP_SessionManager::AddSession(RTP_Session * session)
 
   PTRACE(2, "RTP\tAdding session " << *session);
   sessions.SetAt(session->GetSessionID(), session);
+
+  // The following is the mutex.Signal() that was not done in the UseSession()
   mutex.Signal();
 }
 
@@ -1238,7 +1300,7 @@ void RTP_SessionManager::ReleaseSession(unsigned sessionID)
   if (sessions.Contains(sessionID)) {
     if (sessions[sessionID].DecrementReference()) {
       PTRACE(3, "RTP\tDeleting session " << sessionID);
-      sessions[sessionID].SetJitterBufferSize(0);
+      sessions[sessionID].SetJitterBufferSize(0, 0);
       sessions.SetAt(sessionID, NULL);
     }
   }
@@ -1249,7 +1311,7 @@ void RTP_SessionManager::ReleaseSession(unsigned sessionID)
 
 RTP_Session * RTP_SessionManager::GetSession(unsigned sessionID) const
 {
-  PWaitAndSignal wait(((RTP_SessionManager*)this)->mutex);
+  PWaitAndSignal wait(mutex);
   if (!sessions.Contains(sessionID))
     return NULL;
 
@@ -1355,6 +1417,9 @@ BOOL RTP_UDP::Open(PIPSocket::Address _localAddress, WORD portBase, WORD portMax
   shutdownRead = FALSE;
   shutdownWrite = FALSE;
 
+  if (canonicalName.Find('@') == P_MAX_INDEX)
+    canonicalName += '@' + GetLocalHostName();
+
   PTRACE(2, "RTP_UDP\tSession " << sessionID
          << " created: data=" << dataSocket.GetPort()
          << " control=" << controlSocket.GetPort()
@@ -1392,7 +1457,7 @@ PString RTP_UDP::GetLocalHostName()
 
 BOOL RTP_UDP::SetRemoteSocketInfo(PIPSocket::Address address, WORD port, BOOL isDataPort)
 {
-  PTRACE(3, "RTP_UDP\tSetRemoteSocketInfo: "
+  PTRACE(3, "RTP_UDP\tSetRemoteSocketInfo: session=" << sessionID << ' '
          << (isDataPort ? "data" : "control") << " channel, "
             "new=" << address << ':' << port << ", "
             "local=" << localAddress << ':' << localDataPort << '-' << localControlPort << ", "
@@ -1443,7 +1508,8 @@ BOOL RTP_UDP::ReadData(RTP_DataFrame & frame)
       case -1 :
         switch (ReadDataPDU(frame)) {
           case e_ProcessPacket :
-            return TRUE;
+            if (!shutdownRead)
+              return TRUE;
           case e_IgnorePacket :
             break;
           case e_AbortTransport :
@@ -1482,7 +1548,8 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
       if (remoteTransmitAddress == 0)
         remoteTransmitAddress = addr;
       else if (remoteTransmitAddress != addr) {
-        PTRACE(1, "RTP_UDP\t" << PTRACE_name << " PDU from incorrect host, "
+        PTRACE(1, "RTP_UDP\tSession " << sessionID << ", "
+               << PTRACE_name << " PDU from incorrect host, "
                   " is " << addr << " should be " << remoteTransmitAddress);
         return RTP_Session::e_IgnorePacket;
       }
@@ -1494,7 +1561,8 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(PUDPSocket & socket
   switch (socket.GetErrorNumber()) {
     case ECONNRESET :
     case ECONNREFUSED :
-      PTRACE(2, "RTP_UDP\t" << PTRACE_name << " port on remote not ready.");
+      PTRACE(2, "RTP_UDP\tSession " << sessionID << ", "
+             << PTRACE_name << " port on remote not ready.");
       return RTP_Session::e_IgnorePacket;
 
     default:
