@@ -27,6 +27,42 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: transports.cxx,v $
+ * Revision 1.124  2002/12/23 22:46:06  robertj
+ * Changed gatekeeper discovery so an GRJ does not indicate "discovered".
+ *
+ * Revision 1.123  2002/11/21 06:40:00  robertj
+ * Changed promiscuous mode to be three way. Fixes race condition in gkserver
+ *   which can cause crashes or more PDUs to be sent to the wrong place.
+ *
+ * Revision 1.122  2002/11/12 03:14:18  robertj
+ * Fixed gatekeeper discovery so does IP address translation correctly for
+ *   hosts inside the firewall.
+ *
+ * Revision 1.121  2002/11/10 08:10:44  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.120  2002/11/05 00:31:48  robertj
+ * Prevented a failure to start separate H.245 channel stopping the call until
+ *   after a CONNECT is received and have no audio. At that point no H.245
+ *   is a useless call and we disconnect.
+ *
+ * Revision 1.119  2002/11/01 03:38:18  robertj
+ * More IPv6 fixes, thanks Sbastien Josset.
+ *
+ * Revision 1.118  2002/10/29 08:30:32  robertj
+ * Fixed problem with simultaneous startH245 condition possibly shutting down
+ *   the call under some circumstances.
+ *
+ * Revision 1.117  2002/10/16 06:28:20  robertj
+ * More IPv6 support changes, especially in unambiguising v6 addresses colons
+ *   from the port fields colon, thanks Sebastien Josset
+ *
+ * Revision 1.116  2002/10/08 23:34:30  robertj
+ * Fixed ip v6 usage on H.245 pdu setting.
+ *
+ * Revision 1.115  2002/10/08 13:08:21  robertj
+ * Changed for IPv6 support, thanks Sbastien Josset.
+ *
  * Revision 1.114  2002/08/05 10:03:48  robertj
  * Cosmetic changes to normalise the usage of pragma interface/implementation.
  *
@@ -521,18 +557,46 @@ H323TransportAddress::H323TransportAddress(const PString & str)
 }
 
 
+static PString BuildIP(const PIPSocket::Address & ip, unsigned port)
+{
+  PStringStream str;
+
+  str << IpPrefix;
+
+  if (!ip.IsValid())
+    str << '*';
+  else
+#if P_HAS_IPV6
+  if (ip.GetVersion() == 6)
+    str << '[' << ip << ']';
+  else
+#endif
+    str << ip;
+
+  if (port != 0)
+    str << ':' << port;
+
+  return str;
+}
+
+
 H323TransportAddress::H323TransportAddress(const H225_TransportAddress & transport)
 {
   switch (transport.GetTag()) {
     case H225_TransportAddress::e_ipAddress :
     {
       const H225_TransportAddress_ipAddress & ip = transport;
-      sprintf("%s%u.%u.%u.%u:%u",
-              IpPrefix,
-              ip.m_ip[0], ip.m_ip[1], ip.m_ip[2], ip.m_ip[3],
-              (unsigned)ip.m_port);
+      *this = BuildIP(PIPSocket::Address(ip.m_ip.GetSize(), ip.m_ip.GetValue()), ip.m_port);
       break;
     }
+#if P_HAS_IPV6
+    case H225_TransportAddress::e_ip6Address :
+    {
+      const H225_TransportAddress_ip6Address & ip = transport;
+      *this = BuildIP(PIPSocket::Address(ip.m_ip.GetSize(), ip.m_ip.GetValue()), ip.m_port);
+      break;
+    }
+#endif
   }
 }
 
@@ -543,16 +607,21 @@ H323TransportAddress::H323TransportAddress(const H245_TransportAddress & transpo
     case H245_TransportAddress::e_unicastAddress :
     {
       const H245_UnicastAddress & unicast = transport;
-      switch (transport.GetTag()) {
+      switch (unicast.GetTag()) {
         case H245_UnicastAddress::e_iPAddress :
         {
           const H245_UnicastAddress_iPAddress & ip = unicast;
-          sprintf("%s%u.%u.%u.%u:%u",
-                  IpPrefix,
-                  ip.m_network[0], ip.m_network[1], ip.m_network[2], ip.m_network[3],
-                  (unsigned)ip.m_tsapIdentifier);
+          *this = BuildIP(PIPSocket::Address(ip.m_network.GetSize(), ip.m_network.GetValue()), ip.m_tsapIdentifier);
           break;
         }
+#if P_HAS_IPV6
+        case H245_UnicastAddress::e_iP6Address :
+        {
+          const H245_UnicastAddress_iP6Address & ip = unicast;
+          *this = BuildIP(PIPSocket::Address(ip.m_network.GetSize(), ip.m_network.GetValue()), ip.m_tsapIdentifier);
+          break;
+        }
+#endif
       }
       break;
     }
@@ -561,11 +630,8 @@ H323TransportAddress::H323TransportAddress(const H245_TransportAddress & transpo
 
 
 H323TransportAddress::H323TransportAddress(const PIPSocket::Address & ip, WORD port)
-  : PString(IpPrefix)
 {
-  *this += ip.AsString();
-  if (port != 0)
-    sprintf(":%u", port);
+  *this = BuildIP(ip, port);
 }
 
 
@@ -590,8 +656,19 @@ void H323TransportAddress::Validate()
 BOOL H323TransportAddress::SetPDU(H225_TransportAddress & pdu) const
 {
   PIPSocket::Address ip;
-  WORD port = H323ListenerTCP::DefaultSignalPort;
+  WORD port = H323EndPoint::DefaultTcpPort;
   if (GetIpAndPort(ip, port)) {
+#if P_HAS_IPV6
+    if (ip.GetVersion() == 6) {
+      pdu.SetTag(H225_TransportAddress::e_ip6Address);
+      H225_TransportAddress_ip6Address & addr = pdu;
+      for (PINDEX i = 0; i < ip.GetSize(); i++)
+        addr.m_ip[i] = ip[i];
+      addr.m_port = port;
+      return TRUE;
+    }
+#endif
+
     pdu.SetTag(H225_TransportAddress::e_ipAddress);
     H225_TransportAddress_ipAddress & addr = pdu;
     for (PINDEX i = 0; i < 4; i++)
@@ -600,7 +677,6 @@ BOOL H323TransportAddress::SetPDU(H225_TransportAddress & pdu) const
     return TRUE;
   }
 
-
   return FALSE;
 }
 
@@ -613,8 +689,19 @@ BOOL H323TransportAddress::SetPDU(H245_TransportAddress & pdu) const
     pdu.SetTag(H245_TransportAddress::e_unicastAddress);
 
     H245_UnicastAddress & unicast = pdu;
-    unicast.SetTag(H245_UnicastAddress::e_iPAddress);
 
+#if P_HAS_IPV6
+    if (ip.GetVersion() == 6) {
+      unicast.SetTag(H245_UnicastAddress::e_iP6Address);
+      H245_UnicastAddress_iP6Address & addr = unicast;
+      for (PINDEX i = 0; i < ip.GetSize(); i++)
+        addr.m_network[i] = ip[i];
+      addr.m_tsapIdentifier = port;
+      return TRUE;
+    }
+#endif
+
+    unicast.SetTag(H245_UnicastAddress::e_iPAddress);
     H245_UnicastAddress_iPAddress & addr = unicast;
     for (PINDEX i = 0; i < 4; i++)
       addr.m_network[i] = ip[i];
@@ -650,37 +737,57 @@ BOOL H323TransportAddress::GetIpAddress(PIPSocket::Address & ip) const
 }
 
 
+static BOOL SplitAddress(const PString & addr, PString & host, PString & service)
+{
+  if (strncmp(addr, IpPrefix, 3) != 0) {
+    PTRACE(2, "H323\tUse of non IP transport address: \"" << addr << '"');
+    return FALSE;
+  }
+
+  PINDEX lastChar = addr.GetLength()-1;
+  if (addr[lastChar] == '+')
+    lastChar--;
+
+  PINDEX bracket = addr.FindLast(']');
+  if (bracket == P_MAX_INDEX)
+    bracket = 0;
+
+  PINDEX colon = addr.Find(':', bracket);
+  if (colon == P_MAX_INDEX)
+    host = addr(3, lastChar);
+  else {
+    host = addr.Mid(3, colon-3);
+    service = addr.Mid(colon+1, lastChar);
+  }
+
+  return TRUE;
+}
+
+
 BOOL H323TransportAddress::GetIpAndPort(PIPSocket::Address & ip,
                                         WORD & port,
                                         const char * proto) const
 {
-  if (strncmp(theArray, IpPrefix, 3) != 0) {
-    PTRACE(2, "H323\tUse of non IP transport address: \"" << *this << '"');
-    return FALSE;
-  }
-
-  PINDEX lastChar = GetLength()-1;
-  if ((*this)[lastChar] == '+')
-    lastChar--;
-
   PString host, service;
-  PINDEX colon = Find(':', 3);
-  if (colon == P_MAX_INDEX)
-    host = (*this)(3, lastChar);
-  else {
-    host = Mid(3, colon-3);
-    service = Mid(colon+1, lastChar);
-    if (service == "*")
-      port = 0;
-    else
-      port = PIPSocket::GetPortByService(proto, service);
-  }
+  if (!SplitAddress(*this, host, service))
+    return FALSE;
 
-  if (host.IsEmpty() || (port == 0 && service != "*")) {
+  if (host.IsEmpty()) {
     PTRACE(2, "H323\tIllegal IP transport address: \"" << *this << '"');
     return FALSE;
   }
 
+  if (service == "*")
+    port = 0;
+  else {
+    if (!service)
+      port = PIPSocket::GetPortByService(proto, service);
+    if (port == 0) {
+      PTRACE(2, "H323\tIllegal IP transport port/service: \"" << *this << '"');
+      return FALSE;
+    }
+  }
+
   if (host == "*") {
     ip = INADDR_ANY;
     return TRUE;
@@ -696,16 +803,10 @@ BOOL H323TransportAddress::GetIpAndPort(PIPSocket::Address & ip,
 
 PString H323TransportAddress::GetHostName() const
 {
-  if (strncmp(theArray, IpPrefix, 3) != 0)
+  PString host, service;
+  if (!SplitAddress(*this, host, service))
     return *this;
 
-  PString host;
-  PINDEX colon = Find(':', 3);
-  if (colon == P_MAX_INDEX)
-    host = Mid(3);
-  else
-    host = Mid(3, colon-3);
-
   PIPSocket::Address ip;
   if (PIPSocket::GetHostAddress(host, ip))
     return ip.AsString();
@@ -724,7 +825,7 @@ H323Listener * H323TransportAddress::CreateListener(H323EndPoint & endpoint) con
    */
 
   PIPSocket::Address ip;
-  WORD port = H323ListenerTCP::DefaultSignalPort;
+  WORD port = H323EndPoint::DefaultTcpPort;
   if (GetIpAndPort(ip, port))
     return new H323ListenerTCP(endpoint, ip, port, theArray[GetLength()-1] != '+');
 
@@ -936,11 +1037,17 @@ void H323Transport::SetUpTransportPDU(H245_TransportAddress & /*pdu*/,
 }
 
 
-void H323Transport::SetPromiscuous(BOOL /*promiscuous*/)
+void H323Transport::SetPromiscuous(PromisciousModes /*promiscuous*/)
 {
 }
 
 
+H323TransportAddress H323Transport::GetLastReceivedAddress() const
+{
+  return GetRemoteAddress();
+}
+
+
 H323Transport * H323Transport::CreateControlChannel(H323Connection & /*connection*/)
 {
   PAssertAlways(PUnimplementedFunction);
@@ -1039,16 +1146,7 @@ H323Transport * H323ListenerTCP::Accept(const PTimeInterval & timeout)
 
 H323TransportAddress H323ListenerTCP::GetTransportAddress() const
 {
-  PString addr;
-
-  if (localAddress == INADDR_ANY)
-    addr = "*";
-  else
-    addr = localAddress.AsString();
-
-  addr.sprintf(":%u", listener.GetPort());
-
-  return addr;
+  return H323TransportAddress(localAddress, listener.GetPort());
 }
 
 
@@ -1064,30 +1162,21 @@ static void AppendTransportAddress(H323EndPoint & endpoint,
   if (associatedTransport.GetRemoteAddress().GetIpAddress(remoteIP))
     endpoint.TranslateTCPAddress(addr, remoteIP);
 
-  H225_TransportAddress_ipAddress pduAddr;
-  PINDEX i;
-  for (i = 0; i < 4; i++)
-    pduAddr.m_ip[i] = addr[i];
-  pduAddr.m_port = port;
+  H323TransportAddress transAddr(addr, port);
+  H225_TransportAddress pduAddr;
+  transAddr.SetPDU(pduAddr);
 
   PINDEX lastPos = pdu.GetSize();
 
-  // Check for already have had that IP address.
-  for (i = 0; i < lastPos; i++) {
-    H225_TransportAddress & taddr = pdu[i];
-    if (taddr.GetTag() == H225_TransportAddress::e_ipAddress) {
-      H225_TransportAddress_ipAddress & ipAddr = taddr;
-      if (ipAddr == pduAddr)
-        return;
-    }
+  // Check for already have had that address.
+  for (PINDEX i = 0; i < lastPos; i++) {
+    if (pdu[i] == pduAddr)
+      return;
   }
 
   // Put new listener into array
   pdu.SetSize(lastPos+1);
-  H225_TransportAddress & taddr = pdu[lastPos];
-
-  taddr.SetTag(H225_TransportAddress::e_ipAddress);
-  (H225_TransportAddress_ipAddress &)taddr = pduAddr;
+  pdu[lastPos] = pduAddr;
 }
 
 
@@ -1122,7 +1211,7 @@ void H323ListenerTCP::SetUpTransportPDU(H225_ArrayOf_TransportAddress & pdu,
 
   for (i = 0; i < interfaces.GetSize(); i++) {
     PIPSocket::Address ipAddr = interfaces[i].GetAddress();
-    if (ipAddr != 0 && ipAddr != firstAddress && ipAddr != PIPSocket::Address()) // Ignore 127.0.0.1
+    if (ipAddr != 0 && ipAddr != firstAddress && !ipAddr.IsLoopback()) // Ignore 127.0.0.1
       AppendTransportAddress(endpoint, associatedTransport, ipAddr, port, pdu);
   }
 }
@@ -1131,8 +1220,6 @@ void H323ListenerTCP::SetUpTransportPDU(H225_ArrayOf_TransportAddress & pdu,
 BOOL H323ListenerTCP::SetUpTransportPDU(H245_TransportAddress & pdu,
                                         const H323Transport & associatedTransport)
 {
-  WORD port = listener.GetPort();
-
   if (localAddress != INADDR_ANY)
     return GetTransportAddress().SetPDU(pdu);
 
@@ -1140,15 +1227,8 @@ BOOL H323ListenerTCP::SetUpTransportPDU(H245_TransportAddress & pdu,
   if (!associatedTransport.GetLocalAddress().GetIpAddress(addressOfExistingInterface))
     return FALSE;
 
-  pdu.SetTag(H245_TransportAddress::e_unicastAddress);
-
-  H245_UnicastAddress & unicast = pdu;
-  unicast.SetTag(H245_UnicastAddress::e_iPAddress);
-
-  H245_UnicastAddress_iPAddress & addr = unicast;
-  for (PINDEX i = 0; i < 4; i++)
-    addr.m_network[i] = addressOfExistingInterface[i];
-  addr.m_tsapIdentifier = port;
+  H323TransportAddress transAddr(addressOfExistingInterface, listener.GetPort());
+  transAddr.SetPDU(pdu);
   return TRUE;
 }
 
@@ -1191,44 +1271,34 @@ H323TransportAddress H323TransportIP::GetRemoteAddress() const
 
 BOOL H323TransportIP::IsCompatibleTransport(const H225_TransportAddress & pdu) const
 {
-  return pdu.GetTag() == H225_TransportAddress::e_ipAddress;
+  return pdu.GetTag() == H225_TransportAddress::e_ipAddress
+#if P_HAS_IPV6
+            || pdu.GetTag() == H225_TransportAddress::e_ip6Address
+#endif
+         ;
 }
 
 
 void H323TransportIP::SetUpTransportPDU(H225_TransportAddress & pdu, BOOL localTsap) const
 {
-  pdu.SetTag(H225_TransportAddress::e_ipAddress);
-  H225_TransportAddress_ipAddress & pduAddr = pdu;
-
-  PIPSocket::Address ipAddr;
+  H323TransportAddress transAddr;
   if (!localTsap) 
-    ipAddr = remoteAddress;
+    transAddr = H323TransportAddress(remoteAddress, remotePort);
   else {
-    ipAddr = localAddress;
+    PIPSocket::Address ipAddr = localAddress;
     endpoint.TranslateTCPAddress(ipAddr, remoteAddress);
+    transAddr = H323TransportAddress(ipAddr, localPort);
   }
- 
-  for (PINDEX i = 0; i < 4; i++)
-    pduAddr.m_ip[i] = ipAddr[i];
-  pduAddr.m_port = localTsap ? localPort : remotePort;
+
+  transAddr.SetPDU(pdu);
 }
 
 
 void H323TransportIP::SetUpTransportPDU(H245_TransportAddress & pdu, unsigned port) const
 {
-  pdu.SetTag(H245_TransportAddress::e_unicastAddress);
-
-  H245_UnicastAddress & unicast = pdu;
-  unicast.SetTag(H245_UnicastAddress::e_iPAddress);
-
-  H245_UnicastAddress_iPAddress & addr = unicast;
-
   PIPSocket::Address ipAddr = localAddress;
   endpoint.TranslateTCPAddress(ipAddr, remoteAddress);
 
-  for (PINDEX i = 0; i < 4; i++)
-    addr.m_network[i] = ipAddr[i];
-
   switch (port) {
     case UseLocalTSAP :
       port = localPort;
@@ -1237,7 +1307,9 @@ void H323TransportIP::SetUpTransportPDU(H245_TransportAddress & pdu, unsigned po
       port = remotePort;
       break;
   }
-  addr.m_tsapIdentifier = port;
+
+  H323TransportAddress transAddr(ipAddr, (WORD)port);
+  transAddr.SetPDU(pdu);
 }
 
 
@@ -1246,7 +1318,7 @@ void H323TransportIP::SetUpTransportPDU(H245_TransportAddress & pdu, unsigned po
 H323TransportTCP::H323TransportTCP(H323EndPoint & end,
                                    PIPSocket::Address binding,
                                    BOOL listen)
-  : H323TransportIP(end, binding, H323ListenerTCP::DefaultSignalPort)
+  : H323TransportIP(end, binding, H323EndPoint::DefaultTcpPort)
 {
   h245listener = NULL;
 
@@ -1472,7 +1544,9 @@ BOOL H323TransportTCP::AcceptControlChannel(H323Connection & connection)
   PTRACE(1, "H225\tAccept for H245 failed: " << h245Socket->GetErrorText());
   delete h245Socket;
 
-  if (connection.FindChannel(RTP_Session::DefaultAudioSessionID, TRUE) == NULL ||
+  if (h245listener->IsOpen() &&
+      connection.IsConnected() &&
+      connection.FindChannel(RTP_Session::DefaultAudioSessionID, TRUE) == NULL &&
       connection.FindChannel(RTP_Session::DefaultAudioSessionID, FALSE) == NULL)
     connection.ClearCall(H323Connection::EndedByTransportFail);
 
@@ -1533,11 +1607,10 @@ static BOOL ListenUDP(PUDPSocket & socket,
 
 H323TransportUDP::H323TransportUDP(H323EndPoint & end,
                                    PIPSocket::Address binding,
-                                   WORD port,
-                                   BOOL promiscuous)
-  : H323TransportIP(end, binding, DefaultRasPort)
+                                   WORD port)
+  : H323TransportIP(end, binding, H225_RAS::DefaultRasUdpPort)
 {
-  promiscuousReads = promiscuous;
+  promiscuousReads = AcceptFromRemoteOnly;
 
   PUDPSocket * udp = new PUDPSocket;
   ListenUDP(*udp, end, binding, port);
@@ -1574,12 +1647,21 @@ BOOL H323TransportUDP::Connect()
 }
 
 
-void H323TransportUDP::SetPromiscuous(BOOL promiscuous)
+void H323TransportUDP::SetPromiscuous(PromisciousModes promiscuous)
 {
   promiscuousReads = promiscuous;
 }
 
 
+H323TransportAddress H323TransportUDP::GetLastReceivedAddress() const
+{
+  if (!lastReceivedAddress)
+    return lastReceivedAddress;
+
+  return H323Transport::GetLastReceivedAddress();
+}
+
+
 BOOL H323TransportUDP::ReadPDU(PBYTEArray & pdu)
 {
   for (;;) {
@@ -1596,15 +1678,23 @@ BOOL H323TransportUDP::ReadPDU(PBYTEArray & pdu)
     WORD port;
 
     socket->GetLastReceiveAddress(address, port);
-    if (promiscuousReads) {
-      remoteAddress = address;
-      remotePort = port;
-      socket->SetSendAddress(remoteAddress, remotePort);
-      return TRUE;
-    }
+    lastReceivedAddress = H323TransportAddress(address, port);
 
-    if (remoteAddress == address)
-      return TRUE;
+    switch (promiscuousReads) {
+      case AcceptFromRemoteOnly :
+        if (remoteAddress == address)
+          return TRUE;
+        break;
+
+      case AcceptFromAnyAutoSet :
+        remoteAddress = address;
+        remotePort = port;
+        socket->SetSendAddress(remoteAddress, remotePort);
+        // fall into next case
+
+      default : //AcceptFromAny
+        return TRUE;
+    }
 
     PTRACE(1, "UDP\tReceived PDU from incorrect host: " << address << ':' << port);
   }
@@ -1626,7 +1716,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
   PTRACE(3, "H225\tStarted gatekeeper discovery of \"" << address << '"');
 
   PIPSocket::Address destAddr = INADDR_BROADCAST;
-  WORD destPort = DefaultRasPort;
+  WORD destPort = H225_RAS::DefaultRasUdpPort;
   if (!address) {
     if (!address.GetIpAndPort(destAddr, destPort, "udp")) {
       PTRACE(2, "RAS\tError decoding address");
@@ -1648,8 +1738,8 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
   PIPSocket::InterfaceTable interfaces;
 
   // See if prebound to interface, only use that if so
-  if (destAddr == PIPSocket::Address()) {
-    PTRACE(3, "RAS\tGatekeeper discovery on 127.0.0.1");
+  if (destAddr.IsLoopback()) {
+    PTRACE(3, "RAS\tGatekeeper discovery on loopback interface");
     localAddress = destAddr;
   }
   else if (localAddress != INADDR_ANY) {
@@ -1681,7 +1771,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
   for (i = 0; i < interfaces.GetSize(); i++) {
     localAddress = interfaces[i].GetAddress();
-    if (localAddress == 0 || (destAddr != localAddress && localAddress == PIPSocket::Address()))
+    if (localAddress == 0 || (destAddr != localAddress && localAddress.IsLoopback()))
       continue;
 
     // Check for already have had that IP address.
@@ -1719,7 +1809,9 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 #endif
 
       // Adjust the PDU to reflect the interface we are writing to.
-      SetUpTransportPDU(grq.m_rasAddress, TRUE);
+      PIPSocket::Address ipAddr = localAddress;
+      endpoint.TranslateTCPAddress(ipAddr, destAddr);
+      H323TransportAddress(ipAddr, localPort).SetPDU(grq.m_rasAddress);
 
       PTRACE(3, "RAS\tGatekeeper discovery on interface: " << localAddress << ':' << localPort);
 
@@ -1757,7 +1849,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
         socket->SetOption(SO_BROADCAST, 1);
 
-        socket->SetSendAddress(INADDR_BROADCAST, MulticastRasPort);
+        socket->SetSendAddress(INADDR_BROADCAST, H225_RAS::DefaultRasMulticastPort);
         writeChannel = socket;
         if (request.Write(*this))
           selection.Append(socket);
@@ -1788,7 +1880,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
   for (i = 0; i < selection.GetSize(); i++) {
     readChannel = &selection[i];
-    promiscuousReads = TRUE;
+    promiscuousReads = AcceptFromAnyAutoSet;
 
     H323RasPDU response;
     if (!response.Read(*this)) {
@@ -1798,6 +1890,14 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
 
     do {
       if (gk.HandleRasPDU(response)) {
+        if (!gk.IsDiscoveryComplete()) {
+          localAddress = originalLocalAddress;
+          localPort = originalLocalPort;
+          promiscuousReads = AcceptFromRemoteOnly;
+          readChannel = NULL;
+          return TRUE;
+        }
+
         PUDPSocket * socket = (PUDPSocket *)readChannel;
         socket->GetLocalAddress(localAddress, localPort);
         readChannel = NULL;
@@ -1806,7 +1906,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
           sockets.Remove(socket);
           sockets.AllowDeleteObjects();
 
-          promiscuousReads = FALSE;
+          promiscuousReads = AcceptFromRemoteOnly;
 
           PTRACE(2, "RAS\tGatekeeper discovered at: "
                  << remoteAddress << ':' << remotePort
@@ -1821,7 +1921,7 @@ BOOL H323TransportUDP::DiscoverGatekeeper(H323Gatekeeper & gk,
   PTRACE(2, "RAS\tGatekeeper discovery failed");
   localAddress = originalLocalAddress;
   localPort = originalLocalPort;
-  promiscuousReads = FALSE;
+  promiscuousReads = AcceptFromRemoteOnly;
   readChannel = NULL;
   return FALSE;
 }
