@@ -34,6 +34,9 @@
 /************ Change log
  *
  * $Log: dct.cxx,v $
+ * Revision 1.13  2002/10/24 21:05:26  dereks
+ * Fix compile time warning.
+ *
  * Revision 1.12  2002/05/17 01:47:33  dereks
  * backout the integer maths in the h261 codec.
  *
@@ -69,7 +72,7 @@
 
 #ifndef lint
 //static char rcsid[] =
-//    "@(#) $Header: /home/cvsroot/openh323/src/vic/dct.cxx,v 1.12 2002/05/17 01:47:33 dereks Exp $ (LBL)";
+//    "@(#) $Header: /home/cvsroot/openh323/src/vic/dct.cxx,v 1.13 2002/10/24 21:05:26 dereks Exp $ (LBL)";
 #endif
 
 
@@ -143,6 +146,9 @@ const u_char COLZAG[] = {
 #define FA4 (1.306562965f)
 #define FA5 (0.382683433f)
 
+#ifdef B0
+#undef B0
+#endif
 /*
  * these magic numbers are scaling factors for each coef of the 1-d
  * AA&N DCT.  The scale factor for coef 0 is 1 and coef 1<=n<=7 is
